#!/bin/bash
# Installation KoI
cd koi
# get os
OS="U"
if (cat /etc/issue | grep -i "RASPBIAN")
then
  OS="R"
fi
echo "System:" $OS

# install packages
apt install -y apache2 apache2-utils ssl-cert
apt install -y libapache2-mod-wsgi-py3 python3-dev
apt install -y python3-flask
apt install -y vsftpd
apt install -y pip

if [ "$OS" = "U" ]
then
    apt install -y mysql-server mysql-client python3-mysqldb python3-mysql.connector
else
    apt install -y mariadb-server mariadb-client 
fi

apt install -y python3-reportlab
pip3 install flask_mysqldb
pip3 install svglib
pip3 install python-barcode

if [ "$OS" = "R" ]
then
    apt install -y libxml2-dev
    apt install -y libxslt-dev
    pip3 install mariadb    
fi

# config web server
cp /etc/apache2/apache2.conf apache.conf.backup
echo "ServerName    127.0.0.1" >> /etc/apache2/apache.conf
sed -i "s/Options FollowSymLinks/Options -Indexes -FollowSymLinks -Includes -ExecCGI/g" /etc/apache2/apache2.conf
sed -i "s/Options Indexes FollowSymLinks/Options -Indexes -FollowSymLinks -Includes -ExecCGI/g" /etc/apache2/apache2.conf
sed -i "/^<Directory \/>/a    Deny from all" /etc/apache2/apache2.conf
sed -i "/^<Directory \/>/a    Order deny,allow" /etc/apache2/apache2.conf
sed -i "/^<Directory \/>/a    Require all denied" /etc/apache2/apache2.conf
sed -i "/^<Directory \/var\/www\/>/a    Order deny,allow" /etc/apache2/apache2.conf
sed -i "/^<Directory \/var\/www\/>/a    Require all denied" /etc/apache2/apache2.conf
sed -i "/^<Directory \/var\/www\/>/a    FileEtag None" /etc/apache2/apache2.conf
sed -i "/^<Directory \/var\/www\/>/a    Header always append X-Frame-Options SAMEORIGIN" /etc/apache2/apache2.conf
sed -i '/^<Directory \/var\/www\/>/a    Header set X-XSS-Protection "1; mode=block' /etc/apache2/apache2.conf
sed -i "s|^<Directory \/var\/www\/>|<Directory /var/www/inventory/>|g" /etc/apache2/apache2.conf 
cp /etc/apache2/conf-enabled/security.conf security.conf.backup
sed -i "s/^ServerSignature On/ServerSignature Off/g" /etc/apache2/conf-enabled/security.conf
sed -i "s/^ServerTokens OS/ServerTokens Prod/g" /etc/apache2/conf-enabled/security.conf
cp inventory.conf /etc/apache2/sites-available/
# apache set max connections
sed -i "s/MaxRequestWorkers.*$/MaxRequestWorkers    500/g" /etc/apache2/mods-available/mpm_event.conf
sed -i "s/MaxConnectionsPerChild.*$/MaxConnectionsPerChild    1000/g" /etc/apache2/mods-available/mpm_event.conf

# get first ip address from system
ip=$(hostname --all-ip | cut -f1 -d" ")
# set ip address to virtual host
sed -i "s/{IPADDRESS}/$ip/g" /etc/apache2/sites-available/inventory.conf
cp sslcert.pem /etc/ssl/certs/

a2enmod ssl
a2enmod wsgi
a2enmod rewrite
a2enmod actions
a2enmod headers
a2dismod status
a2ensite inventory.conf
a2dissite 000-default.conf
systemctl restart apache2

# add user for ftp communication
adduser --disabled-password --gecos "" inventory
echo inventory:inventory | chpasswd
mkdir /home/inventory/files
chown inventory:inventory /home/inventory/files
chmod a=rwx /home/inventory/files
chmod +o=rx /home/inventory

# config ftp server
cp vsftpd.key /etc/ssl/private/
cp vsftpd.pem /etc/ssl/certs
sed -i "s/listen=NO/listen=YES/g" /etc/vsftpd.conf
sed -i "s/listen_ipv6=YES/listen_ipv6=NO/g" /etc/vsftpd.conf
sed -i "s/anonymous_enable=YES/anonymous_enable=NO/g" /etc/vsftpd.conf
sed -i "s/#write_enable=YES/write_enable=YES/g" /etc/vsftpd.conf
sed -i "s/#chroot_local_user=YES/chroot_local_user=YES/g" /etc/vsftpd.conf
sed -i "s/#local_umask=022/local_umask=022/g" /etc/vsftpd.conf
# vsftpd with ssl
sed -i "s/rsa_cert_file=.*$/rsa_cert_file=\/etc\/ssl\/certs\/vsftpd.pem/g" /etc/vsftpd.conf
sed -i "s/rsa_private_key_file=.*$/rsa_private_key_file=\/etc\/ssl\/private\/vsftpd.key/g" /etc/vsftpd.conf
sed -i "s/ssl_enable=NO/ssl_enable=YES/g" /etc/vsftpd.conf
echo "require_ssl_reuse=NO" >> /etc/vsftpd.conf
# set chroot and allow only user inventory
echo "user_sub_token=\$USER" >> /etc/vsftpd.conf
echo "local_root=/home/\$USER/files" >> /etc/vsftpd.conf
echo "userlist_enable=YES" >> /etc/vsftpd.conf
echo "userlist_file=/etc/vsftpd.userlist" >> /etc/vsftpd.conf
echo "userlist_deny=NO" >> /etc/vsftpd.conf
echo "allow_writeable_chroot=YES" >> /etc/vsftpd.conf 
echo "pasv_enable=YES" >> /etc/vsftpd.conf 
echo "pasv_min_port=10100" >> /etc/vsftpd.conf 
echo "pasv_max_port=10110" >> /etc/vsftpd.conf 
echo "inventory" > /etc/vsftpd.userlist
# enable read and write file system
sed -i "s/PrivateTmp=.*/PrivateTmp=false/g" /lib/systemd/system/apache2.service
systemctl daemon-reload

# disable ssh login for user inventory
echo "/bin/false" >> /etc/shells
usermod -s /bin/false inventory

# script to set password ftp user
cp setpasswd.sh /usr/sbin/
cat /etc/sudoers sudoers.txt > /tmp/sudoers
mv /tmp/sudoers /etc/sudoers

# install application
mkdir /var/www/inventory
mkdir /var/www/inventory/temp
mkdir /var/www/inventory/attachments
mkdir /var/www/inventory/attacharchiv
cp -a static /var/www/inventory
cp -a templates /var/www/inventory
cp app.wsgi /var/www/inventory
cp inventory.py /var/www/inventory
cp label.py /var/www/inventory
cp itemlist.py /var/www/inventory
ln -s /home/inventory/files /var/www/inventory/.
chown -R www-data:www-data /var/www/inventory

# error in svglib
FILE=$(find /usr/local -name "svglib.py")
cp $FILE .
#sed -i "s/from reportlab.graphics.transform import mmult/#from reportlab.graphics.transform import mmult/g" $FILE

# config database
DATABASE=mysql
if [ "$OS" = "R" ]
then
    DATABASE=mariadb
fi
$DATABASE < database.mysql
$DATABASE inventory < inventar.mysql
$DATABASE inventory < initinventar.mysql

# backup
mkdir -p /work/system
mkdir /work/backup
cp backup.sh /work/system
cp clean.sh /work/system
cp crontab /work/system
crontab /work/system/crontab

# firewall
ufw allow ssh
ufw allow http
ufw allow https
ufw allow ftp
ufw allow ftps
ufw allow from any to any proto tcp port 10100:10110
ufw enable
# restart server
shutdown -r now
