#!/usr/bin/python3
# -*- coding: utf-8 -*-

# File:    inventory.py
# Author:  Volker Matheis

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from flask import Flask, url_for, request, render_template, make_response, escape, session, redirect, jsonify, abort, send_file
import werkzeug
import json
import logging
import os
import uuid
from datetime import datetime
from datetime import date
import re
from flask_mysqldb import MySQL
from werkzeug.debug import DebuggedApplication
from werkzeug.utils import secure_filename
from datetime import date
import time
import tempfile
import hashlib
import shutil
import label
import itemlist
import socket

app = Flask(__name__)
app.secret_key = "BOC_Qqi#vlqtw38cT"
# activate debugger
app.debug = True
app.wsgi_app = DebuggedApplication(app.wsgi_app, evalex=True)

app.config["MYSQL_HOST"] = "localhost"
app.config["MYSQL_USER"] = "dbinventory"
app.config["MYSQL_PASSWORD"] = "Inv2vmfree.org"
app.config["MYSQL_DB"] = "inventory"
mysql = MySQL(app)

TempPath = "/var/www/inventory/temp/"
AttachmentPath = "/var/www/inventory/attachments/"
AttachmentArchivPath = "/var/www/inventory/attacharchiv/"
FTPFilePath = "/var/www/inventory/files/"

app.config["UPLOAD_FOLDER"] = TempPath

MaxDate = "2050-01-01"

logginglevel = logging.INFO
#logginglevel = logging.ERROR
logging.basicConfig(filename="/var/www/inventory/inventory.log",
                level=logginglevel,
                style="{",
                format="{asctime} [{levelname:8}] [{funcName:25}] {message}")

# -------------------------------------------------------------    
def encodePassword(password):
    encodepassword = hashlib.sha256(password.encode("utf-8"))
    return encodepassword.hexdigest()

@app.route("/login")
def login():
    logging.info("login")
    session.clear()
    return render_template("login.html", warning="")

@app.route("/loginuser", methods = ["POST"])
def loginuser():
    logging.info("loginuser")
    inputuser =  request.form["username"]
    inputpassword = encodePassword(request.form["password"])
    
    cursor = mysql.connection.cursor()
    values = {}
    values["username"] = inputuser
    cursor.execute("SELECT number, password, admin, disabled, movement FROM user WHERE username= %(username)s", values)
    result = cursor.fetchall()
    login = False
    session.clear()
    for line in result:
        if line[1] == inputpassword and line[3] == 0:
            session["username"] = line[0]
            session["admin"] = False
            if line[2] == 1:
                session["admin"] = True
            login = True
            session["movement"] = False
            if line[4] == 1:
                session["movement"] = True
            login = True
            break;
    cursor.close()
    if login:
        session["login"] = True
        return redirect("/search")
    warning = "Benutzername oder Passwort falsch"
    return render_template("login.html", warning=warning)

# -------------------------------------------------------------
# info
@app.route("/info")
def info():
    logging.info("info")
    session.clear()
    session["username"] = "nologin"
    session["admin"] = False
    session["login"] = False
    session.pop("select", None)
    statuslist = getStatusList()
    categorylist = getCategoryList()
    detaillist = getDetailList()
    locationlist = getLocationList()
    return render_template("search.html", admin=session["admin"], login=session["login"], statuslist=statuslist, categorylist=categorylist, detaillist=detaillist, locationlist=locationlist, info="", warning="")

# -------------------------------------------------------------
# search
@app.route("/search")
def search():
    logging.info("search")
    checkSession()
    session.pop("select", None)
    statuslist = getStatusList()
    categorylist = getCategoryList()
    detaillist = getDetailList()
    locationlist = getLocationList()
    return render_template("search.html", movement=session["movement"], admin=session["admin"], login=session["login"], statuslist=statuslist, categorylist=categorylist, detaillist=detaillist, locationlist=locationlist, info="", warning="")

# get status description
def getStatusDescription(value):
    try:
        values = {}
        values["number"] = value
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT description FROM status WHERE number= %(number)s", values)
        result = cursor.fetchall()
        for line in result:
            return line[0]
        cursor.close()
    except Exception as error:
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return ""
    
# get category description
def getCategoryDescription(value):
    try:
        values = {}
        values["number"] = value
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT description FROM category WHERE number= %(number)s", values)
        result = cursor.fetchall()
        for line in result:
            return line[0]
        cursor.close()
    except Exception as error:
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return ""

# get detail description
def getDetailDescription(value):
    try:
        values = {}
        values["number"] = value
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT description FROM detail WHERE number= %(number)s", values)
        result = cursor.fetchall()
        for line in result:
            return line[0]
        cursor.close()
    except Exception as error:
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return ""

def getLocationDescription(value):
    try:
        values = {}
        values["number"] = value
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT description FROM location WHERE number= %(number)s", values)
        result = cursor.fetchall()
        for line in result:
            return line[0]
        cursor.close()
    except Exception as error:
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return ""

# list inventory
@app.route("/inventory", methods=["POST"])
def inventory():
    logging.info("inventory")
    checkSession()
    andorlist = request.form.getlist("searchandor")
    andor = " " + andorlist[0] + " "
    values = {}
    items = ""
    statement = ""
    order = ""
    orderstatement = ""
    number = request.form["number"]
    numberfrom = request.form["numberfrom"]
    numberto = request.form["numberto"]
    if number != "":
        values["number"] = number
        items = "number = %(number)s"
        statement = andor
        order = "number"
        orderstatement = ", "
    if numberfrom != "":
        values["numberfrom"] = numberfrom
        items = "number >= %(numberfrom)s"
        statement = andor
        order = "number"
        orderstatement = ", "
    if numberto != "":
        values["numberto"] = numberto
        items = "number <= %(numberto)s"
        if numberfrom != "":
            items = "number >= %(numberfrom)s AND number <= %(numberto)s"
        statement = andor
        order = "number"
        orderstatement = ", "
    description = request.form["description"]
    if description != "":
        description = "%" + description + "%"
        values["description"] = description
        items = items + statement + "description LIKE %(description)s"
        statement = andor
        order = order + orderstatement + "description"
        orderstatement = ", "
    serialnumber = request.form["serialnumber"]
    if serialnumber != "":
        serialnumber = "%" + serialnumber + "%"
        values["serialnumber"] = serialnumber
        items = items + statement + "serialnumber LIKE %(serialnumber)s"
        statement = andor
        order = order + orderstatement + "serialnumber"
        orderstatement = ", "    
    try:
        category = request.form["category"]
        if category != "-":
            values["category"] = getCategoryNumber(category)
            items = items + statement + "category = %(category)s"
            statement = andor 
            order = order + orderstatement + "category"
            orderstatement = ", "
    except Exception as error:
        pass
    try:
        detail = request.form["detail"]
        if detail != "-":
            values["detail"] = getDetailNumber(detail)
            items = items + statement + "detail = %(detail)s"
            statement = andor 
            order = order + orderstatement + "detail"
            orderstatement = ", "
    except Exception as error:
        pass
    try:
        status = request.form["status"]
        if status != "-":
            values["status"] = getStatusNumber(status)
            items = items + statement + "status = %(status)s"
            statement = andor 
            order = order + orderstatement + "status"
            orderstatement = ", "
    except Exception as error:
        pass
    try:
        location = request.form["location"]
        if location != "-":
            values["location"] = getLocationNumber(location)
            items = items + statement + "status = %(location)s"
            statement = andor 
            order = order + orderstatement + "location"
            orderstatement = ", "
    except Exception as error:
        pass
    place = request.form["place"]
    if place != "":
        place = "%" + place + "%"
        values["place"] = place
        items = items + statement + "place LIKE %(place)s"
        statement = andor
        order = order + orderstatement + "place"
        orderstatement = ", "
    invoice = request.form["invoice"]
    if invoice != "":
        invoice = "%" + invoice + "%"
        values["invoice"] = invoice
        items = items + statement + "invoice LIKE %(invoice)s"
        statement = andor
        order = order + orderstatement + "invoice"
        orderstatement = ", "
    
    try:
        cursor = mysql.connection.cursor()
        if "select" in session:
            selectcommand = session["select"]
            if "selectvalues" in session:
                values = session["selectvalues"]
                cursor.execute(selectcommand, values)
            else:
                cursor.execute(selectcommand)
        else:
            if items == "":
                selectcommand = "SELECT number, description, serialnumber, category, detail, status, location, place, invoice, quantity, removedquantity FROM item"
                cursor.execute(selectcommand)
            else:
                selectcommand = "SELECT number, description, serialnumber, category, detail, status, location, place, invoice, quantity, removedquantity FROM item WHERE " + items + " ORDER BY " + order
                cursor.execute(selectcommand, values)
                session["select"] = selectcommand
                session["selectvalues"] = values
        
        result = cursor.fetchall()
        inventorylist = []
        for line in result:
            inventory = []
            inventory.append(line[0])
            inventory.append(line[1])
            inventory.append(line[2])
            inventory.append(getCategoryDescription(line[3]))
            inventory.append(getDetailDescription(line[4]))
            inventory.append(getStatusDescription(line[5]))
            inventory.append(getLocationDescription(line[6]))
            inventory.append(line[7])
            inventory.append(line[8])
            inventory.append(line[9])
            inventory.append(line[10])
            inventorylist.append(inventory)
        cursor.close()
    except Exception as error:
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return render_template("inventory.html", inventorylist=inventorylist, movement=session["movement"], admin=session["admin"], login=session["login"], info="", warning="")

@app.route("/inventoryback", methods=["POST"]) 
def inventoryback():
    try:
        cursor = mysql.connection.cursor()
        if items == "":
            selectcommand = "SELECT number, description, serialnumber, category, detail, status, location, place, invoice, quantity, removedquantity FROM item"
            cursor.execute(selectcommand)
        else:
            selectcommand = "SELECT number, description, serialnumber, category, detail, status, location, place, invoice, quantity, removedquantity FROM item WHERE " + items + " ORDER BY " + order
            cursor.execute(selectcommand, values)
        
        result = cursor.fetchall()
        inventorylist = []
        for line in result:
            inventory = []
            inventory.append(line[0])
            inventory.append(line[1])
            inventory.append(line[2])
            inventory.append(getCategoryDescription(line[3]))
            inventory.append(getDetailDescription(line[4]))
            inventory.append(getStatusDescription(line[5]))
            inventory.append(getLocationDescription(line[6]))
            inventory.append(line[7])
            inventory.append(line[8])
            inventory.append(line[9])
            inventory.append(line[10])
            inventorylist.append(inventory)
        cursor.close()
    except Exception as error:
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return render_template("inventory.html", inventorylist=inventorylist, movement=session["movement"], admin=session["admin"], info="", warning="")
    
# get category list
def getCategoryList():
    categorylist = []
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT description FROM category ORDER BY description")
        result = cursor.fetchall()
        for line in result:
            category = []
            category.append(line[0])
            categorylist.append(category)
        cursor.close()
    except Exception as error:
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return categorylist

# get detail list
def getDetailList():
    detaillist = []
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT description FROM detail ORDER BY description")
        result = cursor.fetchall()
        for line in result:
            detail = []
            detail.append(line[0])
            detaillist.append(detail)
        cursor.close()
    except Exception as error:
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return detaillist

# get location list
def getLocationList():
    locationlist = []
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT description FROM location ORDER BY description")
        result = cursor.fetchall()
        for line in result:
            location = []
            location.append(line[0])
            locationlist.append(location)
        cursor.close()
    except Exception as error:
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return locationlist

# get status list
def getStatusList():
    statuslist = []
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT description FROM status ORDER BY description")
        result = cursor.fetchall()
        for line in result:
            status = []
            status.append(line[0])
            statuslist.append(status)
        cursor.close()
    except Exception as error:
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return statuslist

# new inventory
@app.route("/add", methods=["GET"])
def add():
    logging.info("add")
    checkSession()
    checkBooking()
    info = request.args.get("info")
    if info == None:
        info = ""
    else:
        info = info = "Inventarnummer: " + info
    statuslist = getStatusList()
    categorylist = getCategoryList()
    detaillist = getDetailList()
    locationlist = getLocationList()
    lastinspection = date.today()
    outgoing = "2100-01-01"
    return render_template("inventorynew.html", statuslist=statuslist, categorylist=categorylist, detaillist=detaillist, locationlist=locationlist, lastinspection=lastinspection, outgoing=outgoing, admin=session["admin"], info=info, warning="")

# get Category
def getCategoryNumber(value):
    try:
        values = {}
        values["description"] = value
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number FROM category WHERE description= %(description)s", values)
        result = cursor.fetchall()
        for line in result:
            return line[0]
        cursor.close()
    except Exception as error:
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return 0
    
# get detail
def getDetailNumber(value):
    try:
        values = {}
        values["description"] = value
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number FROM detail WHERE description= %(description)s", values)
        result = cursor.fetchall()
        for line in result:
            return line[0]
        cursor.close()
    except Exception as error:
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return 0

# get Location
def getLocationNumber(value):
    try:
        values = {}
        values["description"] = value
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number FROM location WHERE description= %(description)s", values)
        result = cursor.fetchall()
        for line in result:
            return line[0]
        cursor.close()
    except Exception as error:
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return 0

# get Status
def getStatusNumber(value):
    try:
        values = {}
        values["description"] = value
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number FROM status WHERE description = %(description)s", values)
        result = cursor.fetchall()
        for line in result:
            return line[0]
        cursor.close()
    except Exception as error:
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return 0

def getUser():
    values = {}
    values["number"] = session["username"]
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT username FROM user WHERE number = %(number)s", values)
    result = cursor.fetchall()
    for line in result:
        return line[0]
    cursor.close()
    return None

def writehistory(number, cursor):
    logging.info("writehistory")
    try:
        valuesselect = {}
        valuesselect["number"] = number
        #cursor = mysql.connection.cursor()
        cursor.execute("SELECT number, description, serialnumber, keynumber, category, detail, status, location, place, price, quantity, invoice, lastinspection, incoming, outgoing, changed, user, attachment, notice, lastelectricaltest FROM item WHERE number= %(number)s", valuesselect)
        result = cursor.fetchall()
        valueshistory = []
        for line in result:
            valueshistory.append(line[0])
            items = "itemnumber, " 
            placeholder = "%s, "
            valueshistory.append(line[1])
            items = items + "description, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[2])
            items = items + "serialnumber, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[3])
            items = items + "keynumber, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[4])
            items = items + "category, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[5])
            items = items + "detail, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[6])
            items = items + "status, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[7])
            items = items + "location, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[8])
            items = items + "place, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[9])
            items = items + "price, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[10])
            items = items + "quantity, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[11])
            items = items + "invoice, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[12])
            items = items + "lastinspection, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[13])
            items = items + "incoming, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[14])
            items = items + "outgoing, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[15])
            items = items + "changed, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[16])
            items = items + "user, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[17])
            items = items + "attachment, "
            placeholder = placeholder + "%s, "
            valueshistory.append(line[18])
            items = items + "notice, "
            placeholder = placeholder + "%s,"
            valueshistory.append(line[19])
            items = items + "lastelectricaltest"
            placeholder = placeholder + "%s"
        cursor.execute("INSERT INTO history (" + items + ") VALUES (" + placeholder + ")", valueshistory)
        number = cursor.lastrowid
        logging.info(number)
        if line[17] != "":
            copyAttachment(number, line[17], cursor)
        mysql.connection.commit()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))

def copyAttachment(number, filenamein, cursor):
    logging.info("copyAttachment")
    filenameout = str(number) + "_" + filenamein
    shutil.copy(AttachmentPath + filenamein, AttachmentArchivPath + filenameout)
    try:
        values = []
        values.append(number)
        cursor.execute("UPDATE history SET attachment = '" + filenameout  + "' WHERE number = %s", values)
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    
# get next item number
def getNextItemNumber():
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("LOCK TABLES counter WRITE");
        cursor.execute("SELECT item FROM counter WHERE number = 1")
        result = cursor.fetchall()
        for line in result:
            item = int(line[0])
        item = item + 1
        values = []
        values.append(item)
        cursor.execute("UPDATE counter SET item = %s WHERE number = 1", values)
        cursor.execute("UNLOCK TABLES");
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return item
 
def getItem(number):
    logging.info("getItem")
    values = {}
    values["number"] = number
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number, description, serialnumber, keynumber, category, detail, status, location, place, price, quantity, invoice, lastinspection, incoming, outgoing, changed, user, updatecounter, attachment, notice, lastelectricaltest, removedquantity FROM item WHERE number= %(number)s", values)
        result = cursor.fetchall()
        for line in result:
            if (line[0] is None):
                number = "0"
            else:
                number = str(line[0])
            if (line[1] is None):
                description = ""
            else:
                description = line[1]
            if (line[2] is None):
                serialnumber = ""
            else:
                serialnumber = line[2]
            if (line[3] is None):
                keynumber = ""
            else:
                keynumber = line[3]
            if (line[4] is None):
                category = ""
                categorydb = ""
            else:
                category = getCategoryDescription(line[4])
                categorydb = category
            if (line[5] is None):
                detail = ""
                detaildb = ""
            else:
                detail = getDetailDescription(line[5])
                detaildb = detail
            if (line[6] is None):
                status = ""
                statusdb = ""
            else:
                status = getStatusDescription(line[6])
                statusdb = status
            if (line[7] is None):
                location = ""
                locationdb = ""
            else:
                location = getLocationDescription(line[7])
                locationdb = location
            if (line[8] is None):
                place = ""
            else:
                place = line[8]
            if (line[9] is None):
                price = ""
            else:
                price = line[9]
            if (line[10] is None):
                quantity = ""
            else:
                quantity = line[10]
            if (line[11] is None):
                invoice = ""
            else:
                invoice = line[11]
            if (line[12] is None):
                lastinspection = ""
            else:
                lastinspection = line[12]
            if (line[13] is None):
                incoming = ""
            else:
                incoming = line[13]
            if (line[14] is None):
                outgoing = ""
            else:
                outgoing = line[14]
            if (line[15] is None):
                changed = ""
            else:
                changed = line[15]
            if (line[16] is None):
                user = ""
            else:
                user = line[16]
            if (line[17] is None):
                updatecounter = 0
            else:
                updatecounter = line[17]
            if (line[18] is None):
                attachment = ""
            else:
                attachment = line[18]
            if (line[19] is None):
                notice = ""
            else:
                notice = line[19]
            if (line[20] is None):
                lastelectricaltest = ""
            else:
                lastelectricaltest = line[20]
                
            if (line[21] is None):
                removedquantity = 0
            else:
                removedquantity = line[21]
        
        cursor.close()
        statuslist = getStatusList()
        categorylist = getCategoryList()
        detaillist = getDetailList()
        locationlist = getLocationList()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return number, description, serialnumber, keynumber, categorydb, detaildb, statusdb, locationdb, place, invoice, price, quantity,  lastinspection, incoming, outgoing, changed, user, updatecounter, statuslist, categorylist, detaillist, locationlist, attachment, notice, lastelectricaltest, removedquantity

def getItemDetail(number):
    logging.info("getItemDetail")
    values = {}
    values["number"] = number
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number, description, serialnumber, status, location, place, quantity, removedquantity FROM item WHERE number= %(number)s", values)
        result = cursor.fetchall()
        for line in result:
            if (line[0] is None):
                number = "0"
            else:
                number = str(line[0])
            if (line[1] is None):
                description = ""
            else:
                description = line[1]
            if (line[2] is None):
                serialnumber = ""
            else:
                serialnumber = line[2]
            status = getStatusDescription(line[3])
            location = getLocationDescription(line[4])
            if (line[5] is None):
                place = ""
            else:
                place = line[5]
            if (line[6] is None):
                quantity = ""
            else:
                quantity = line[6]
            if (line[7] is None):
                removedquantity = 0
            else:
                removedquantity = line[7]
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return number, description, serialnumber, status, location, place, quantity, removedquantity

def isItem(number):
    logging.info("isItem")
    values = {}
    values["number"] = number
    exists = False
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number FROM item WHERE number= %(number)s", values)
        result = cursor.fetchall()
        for line in result:
            exists = True
        cursor.close()
    except Exception as error:
        exists = False
    return exists
    
def getQuantity(number):
    logging.info("itemQuantity")
    values = {}
    values["number"] = number
    quantity = 0
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT quantity FROM item WHERE number= %(number)s", values)
        result = cursor.fetchall()
        for line in result:
            quantity = int(line[0])
        cursor.close()
    except Exception as error:
        quantity = 0
    return quantity

# edit inventory
@app.route("/inventoryedit", methods=["POST"])
def inventoryedit():
    logging.info("inventoryedit")
    checkSession()

    try:
        value = request.form["edit"]
        action = "edit"
    except:
        pass
    try:
        value = request.form["delete"]
        action = "delete"
    except:
        pass
    try:
        value = request.form["view"]
        action = "view"
    except:
        pass
    try:
        value = request.form["print"]
        action = "print"
    except:
        pass
    try:
        value = request.form["printlist"]
        action = "printlist"
    except:
        pass
    try:
        value = request.form["history"]
        action = "history"
    except:
        pass
    try:
        value = request.form["copy"]
        action = "copy"
    except:
        pass
    try:
        value = request.form["editall"]
        action = "editall"
    except:
        pass
    try:
        value = request.form["attachmentview"]
        action = "attachmentview"
    except:
        pass
    try:
        value = request.form["search"]
        action = "search"
    except:
        pass
    try:
        value = request.form["remove"]
        action = "remove"
    except:
        pass
    try:
        value = request.form["store"]
        action = "store"
    except:
        pass
    
    if action == "search":
        return redirect("/search")
    
    if action == "attachmentview":
        filename = AttachmentPath + request.form["attachment"]
        return send_file(filename, as_attachment=True)
    
    if action == "history":
        number = request.form["history"]
        description, historylist = history(number)
        return render_template("history.html", number=value, description=description, historylist=historylist, admin=session["admin"], info="", warning="")

    if action == "printlist":
        printnumberlist = request.form.getlist("printstate")
        numberlist = ""
        for line in printnumberlist:
            numberlist = numberlist + "," + line
        pdffile = printitemlist(numberlist)
        return send_file(pdffile, as_attachment=True)

    numberlist = ""
    if action == "editall":
        editnumberlist = request.form.getlist("printstate")
        numberlist = ""
        value = ""
        for line in editnumberlist:
            if value == "":
                value = line
            numberlist = numberlist + "," + line

    if action == "print":
        printnumberlist = request.form.getlist("printstate")
        numberlist = ""
        for line in printnumberlist:
            numberlist = numberlist + "," + line
        try:
            cursor = mysql.connection.cursor()
            cursor.execute("SELECT description, defaultlabel FROM label ORDER BY defaultlabel")
            result = cursor.fetchall()
            labelsizelist = []
            for line in result:
                if line[1] == 1:
                    labelsize = line[0]
                else:
                    labelsize = line[0]
                labellist = []
                labellist.append(line[0])
                labelsizelist.append(labellist)
            cursor.close()
        except Exception as error:
            mysql.connection.rollback()
            logging.error(str(error))
            return jsonify("error: " + str(error))
        return render_template("printlabel.html", labelsizelist=labelsizelist, labelsize=labelsize, numberlist=numberlist, admin=session["admin"], info="", warning="")

    if action == "remove":
        number = request.form["remove"]
        number, description, serialnumber, status, location, place, quantity, removedquantity = getItemDetail(number)
        quantity = quantity - removedQuantity(number)
        minmovement = 0
        if quantity > 0:
            minmovement = 1
        listmovement = getMovement(number)
        return render_template("movement.html", state="REMOVE", text="Entnehmen", number=number, description=description, serialnumber=serialnumber, status=status, location=location, place=place, quantity=quantity, minmovement=minmovement, maxmovement=quantity, listmovement=listmovement, movement=session["movement"], admin=session["admin"], info="", warning="")

    if action == "store":
        number = request.form["store"]
        number, description, serialnumber, status, location, place, quantity, removedquantity = getItemDetail(number)
        quantity = removedQuantity(number)
        minmovement = 0
        if quantity > 0:
            minmovement = 1
        listmovement = getMovement(number)
        return render_template("movement.html", state="STORE", text="Zurück", number=number, description=description, serialnumber=serialnumber, status=status, location=location, place=place, quantity=quantity, minmovement=minmovement, maxmovement=quantity, listmovement=listmovement,  movement=session["movement"], admin=session["admin"], info="", warning="")
    
    values = {}
    values["number"] = value
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number, description, serialnumber, keynumber, category, detail, status, location, place, price, quantity, invoice, lastinspection, incoming, outgoing, changed, user, updatecounter, attachment, notice, lastelectricaltest, removedquantity FROM item WHERE number= %(number)s", values)
        result = cursor.fetchall()
        for line in result:
            if (line[0] is None):
                number = "0"
            else:
                number = str(line[0])
            if (line[1] is None):
                description = ""
            else:
                description = line[1]
            if (line[2] is None):
                serialnumber = ""
            else:
                serialnumber = line[2]
            if (line[3] is None):
                keynumber = ""
            else:
                keynumber = line[3]
            if (line[4] is None):
                category = ""
                categorydb = ""
            else:
                category = getCategoryDescription(line[4])
                categorydb = category
            if (line[5] is None):
                detail = ""
                detaildb = ""
            else:
                detail = getDetailDescription(line[5])
                detaildb = detail
            if (line[6] is None):
                status = ""
                statusdb = ""
            else:
                status = getStatusDescription(line[6])
                statusdb = status
            if (line[7] is None):
                location = ""
                locationdb = ""
            else:
                location = getLocationDescription(line[7])
                locationdb = location
            if (line[8] is None):
                place = ""
            else:
                place = line[8]
            if (line[9] is None):
                price = ""
            else:
                price = line[9]
            if (line[10] is None):
                quantity = ""
            else:
                quantity = line[10]
            if (line[11] is None):
                invoice = ""
            else:
                invoice = line[11]
            if (line[12] is None):
                lastinspection = ""
            else:
                lastinspection = line[12]
            if (line[13] is None):
                incoming = ""
            else:
                incoming = line[13]
            if (line[14] is None):
                outgoing = ""
            else:
                outgoing = line[14]
            if (line[15] is None):
                changed = ""
            else:
                changed = line[15]
            if (line[16] is None):
                user = ""
            else:
                user = line[16]
            if (line[17] is None):
                updatecounter = 0
            else:
                updatecounter = line[17]
            if (line[18] is None):
                attachment = ""
            else:
                attachment = line[18]
            if (line[19] is None):
                notice = ""
            else:
                notice = line[19]
            if (line[20] is None):
                lastelectricaltest = ""
            else:
                lastelectricaltest = line[20]
            if (line[21] is None):
                removedquantity = 0
            else:
                removedquantity = line[21]
        
        cursor.close()
        statuslist = getStatusList()
        categorylist = getCategoryList()
        detaillist = getDetailList()
        locationlist = getLocationList()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))

    if action == "delete":
        return render_template("inventorydelete.html", number=number, description=description, serialnumber=serialnumber, keynumber=keynumber, category=category, detail=detail, status=status, location=location, place=place, invoice=invoice, price=price, quantity=quantity, lastinspection=lastinspection, lastelectricaltest=lastelectricaltest, incoming=incoming, outgoing=outgoing, changed=changed, user=user, attachment=attachment, notice=notice, admin=session["admin"], info="", warning="")
    if action == "view":
        listmovement = getMovement(number)
        return render_template("inventoryview.html", number=number, description=description, serialnumber=serialnumber, keynumber=keynumber, category=category, detail=detail, status=status, location=location, place=place, invoice=invoice, price=price, quantity=quantity, lastinspection=lastinspection, lastelectricaltest=lastelectricaltest, incoming=incoming, outgoing=outgoing, changed=changed, user=user, attachment=attachment, notice=notice, removedquantity=removedquantity, listmovement=listmovement, admin=session["admin"], login=session["login"], info="", warning="")
    if action == "new":
        return render_template("inventorynew.html", description=description, serialnumber=serialnumber, keynumber=keynumber, categorydb=categorydb, categorylist=categorylist, detaildb=detaildb, detaillist=detaillist, statusdb=statusdb, statuslist=statuslist, locationdb=locationdb, locationlist=locationlist, place=place, invoice=invoice, price=price, quantity=quantity, lastinspection=lastinspection, lastelectricaltest=lastelectricaltest, incoming=incoming, outgoing=outgoing, changed=changed, user=user, updatecounter=updatecounter, numberlist=numberlist, notice=notice, admin=session["admin"], info="", warning="")
    if action == "copy":
        return render_template("inventorycopy.html", number=number, description=description, serialnumber=serialnumber, keynumber=keynumber, categorydb=categorydb, categorylist=categorylist, detaildb=detaildb, detaillist=detaillist, statusdb=statusdb, statuslist=statuslist, locationdb=locationdb, locationlist=locationlist, place=place, invoice=invoice, price=price, quantity=quantity, lastinspection=lastinspection, lastelectricaltest=lastelectricaltest, incoming=incoming, outgoing=outgoing, changed=changed, user=user, updatecounter=updatecounter, attachment=attachment, numberlist=numberlist, notice=notice, admin=session["admin"], info="", warning="")
    if action == "edit":
        listmovement = getMovement(number)
        return render_template("inventoryedit.html", number=number, description=description, serialnumber=serialnumber, keynumber=keynumber, categorydb=categorydb, categorylist=categorylist, detaildb=detaildb, detaillist=detaillist, statusdb=statusdb, statuslist=statuslist, locationdb=locationdb, locationlist=locationlist, place=place, invoice=invoice, price=price, quantity=quantity, lastinspection=lastinspection, lastelectricaltest=lastelectricaltest, incoming=incoming, outgoing=outgoing, changed=changed, user=user, updatecounter=updatecounter, attachment=attachment, numberlist=numberlist, notice=notice, removedquantity=removedquantity, listmovement=listmovement, admin=session["admin"], info="", warning="")
    if action == "editall":
        if numberlist != "":
            listmovement = getMovement(number)
            return render_template("inventoryedit.html", number=number, description=description, serialnumber=serialnumber, keynumber=keynumber, categorydb=categorydb, categorylist=categorylist, detaildb=detaildb, detaillist=detaillist, statusdb=statusdb, statuslist=statuslist, locationdb=locationdb, locationlist=locationlist, place=place, invoice=invoice, price=price, quantity=quantity, lastinspection=lastinspection, lastelectricaltest=lastelectricaltest, incoming=incoming, outgoing=outgoing, changed=changed, user=user, updatecounter=updatecounter, attachment=attachment, numberlist=numberlist, notice=notice, listmovement=listmovement, admin=session["admin"], info="", warning="")
    return redirect("/search")

def removedQuantity(number):
    logging.info("removedQuantity")
    values = {}
    values["number"] = number
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT removedquantity FROM item WHERE number= %(number)s", values)
    result = cursor.fetchall()
    removed = 0
    for line in result:
        removed = line[0]
    cursor.close()
    return removed

def getMovement(number):
    logging.info("getMovement")
    values = {}
    values["number"] = number
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT user, quantity, movementdate, notice FROM movement WHERE itemnumber= %(number)s ORDER BY movementdate DESC", values)
    result = cursor.fetchall()
    listmovement = []
    for line in result:
        liste = []
        quantity = line[1]
        typ = "zurück"
        if quantity < 0:
            typ = "entnommen"
            quantity = quantity * -1
        liste.append(typ)
        liste.append(quantity)
        datestring = line[2].strftime("%d.%m.%Y %H:%M")
        liste.append(datestring)
        liste.append(line[0])
        liste.append(line[3])
        listmovement.append(liste)
    cursor.close()
    logging.info(listmovement)
    return listmovement 
    
# movement item
@app.route("/movement", methods=["POST"])
def movement():
    logging.info("movement")
    checkSession()
    try:
        value = request.form["search"]
        action = "search"
    except:
        pass
    try:
        state = request.form["remove"]
        action = "remove"
    except:
        pass
    try:
        state = request.form["store"]
        action = "store"
    except:
        pass
    
    if action == "remove" or action == "store":
        number = request.form["number"]
        quantity = int(request.form["movement"])
        notice = ""
        if action == "remove":
            notice = request.form["notice"]
        movementSave(number, quantity, notice, action)
    return redirect("/search")

def movementSave(number, quantity, notice, action):
    logging.info("movementSave")
    values = {}
    values["number"] = number
    values["user"] = getUser()
    logging.info(values["user"])
    datestring = datetime.now().strftime("%Y-%m-%d %H:%M")
    values["movementdate"] = datestring
    values["quantity"] = quantity
    if action == "remove":
        values["quantity"] = quantity * -1
    values["notice"] = notice
    cursor = mysql.connection.cursor()
    cursor.execute("INSERT INTO movement (itemnumber, user, movementdate, quantity, notice) VALUES (%(number)s, %(user)s, %(movementdate)s, %(quantity)s, %(notice)s)", values)
    values = {}
    values["number"] = number
    cursor.execute("SELECT removedquantity FROM item WHERE number = %(number)s", values)
    result = cursor.fetchall()
    for line in result:
        removedquantity = line[0]
    if action == "remove":
        removedquantity = removedquantity + quantity
    if action == "store":
        removedquantity = removedquantity - quantity
    values["removedquantity"] = removedquantity
    cursor.execute("UPDATE item SET removedquantity = %(removedquantity)s WHERE number = %(number)s", values)
    mysql.connection.commit()
    cursor.close()
    
# remove many
@app.route("/removemany", methods=["GET", "POST"])
def removeMany():
    logging.info("removeMany")
    number1 = ""
    number2 = ""
    number3 = ""
    number4 = ""
    number5 = ""
    quantity1 = ""
    quantity2 = ""
    quantity3 = ""
    quantity4 = ""
    quantity5 = ""
    notice1 = ""
    notice2 = ""
    notice3 = ""
    notice4 = ""
    notice5 = ""
    comment1 = ""
    comment2 = ""
    comment3 = ""
    comment4 = ""
    comment5 = ""
    description1 = ""
    description2 = ""
    description3 = ""
    description4 = ""
    description5 = ""
    style1 = ""
    style2 = ""
    style3 = ""
    style4 = ""
    style5 = ""
    
    try:
        number1 = request.form["number1"]
    except:
        pass
    try:
        number2 = request.form["number2"]
    except:
        pass
    try:
        number3 = request.form["number3"]
    except:
        pass
    try:
        number4 = request.form["number4"]
    except:
        pass
    try:
        number5 = request.form["number5"]
    except:
        pass
    try:
        quantity1 = request.form["quantity1"]
    except:
        pass    
    try:
        quantity2 = request.form["quantity2"]
    except:
        pass    
    try:
        quantity3 = request.form["quantity3"]
    except:
        pass    
    try:
        quantity4 = request.form["quantity4"]
    except:
        pass    
    try:
        quantity5 = request.form["quantity5"]
    except:
        pass
    try:
        notice1 = request.form["notice1"]
    except:
        pass
    try:
        notice2 = request.form["notice2"]
    except:
        pass
    try:
        notice3 = request.form["notice3"]
    except:
        pass
    try:
        notice4 = request.form["notice4"]
    except:
        pass
    try:
        notice5 = request.form["notice5"]
    except:
        pass    
    
    action = ""
    try:
        value = request.form["remove"]
        action = "remove"
    except:
        pass
    
    error = False
    
    if action == "remove":
        cursor = mysql.connection.cursor()
        for index in range(1,6):
            var = "number" + str(index)
            try:
                number = request.form[var]
            except:
                number = ""
            if number != "":
                values = {}
                values["number"] = number
                cursor.execute("SELECT quantity, removedquantity, description FROM item WHERE number = %(number)s", values)
                result = cursor.fetchall()
                exists = False
                for line in result:
                    exists = True
                    if index == 1:
                        description1 = line[2]
                    if index == 2:
                        description2 = line[2]
                    if index == 3:
                        description3 = line[2]
                    if index == 4:
                        description4 = line[2]
                    if index == 5:
                        description5 = line[2]
                if not exists:
                    error = True
                    text = "Inventar nicht gefunden"
                    if index == 1:
                        comment1 = text
                        style1 = "background-color: Crimson"
                    if index == 2:
                        comment2 = text
                        style2 = "background-color: Crimson"
                    if index == 3:
                        comment3 = text
                        style3 = "background-color: Crimson"
                    if index == 4:
                        comment4 = text
                        style4 = "background-color: Crimson"
                    if index == 5:
                        comment5 = text
                        style5 = "background-color: Crimson"
                else:
                    quantity = line[0] - line[1]
                    var = "quantity" + str(index)
                    try:
                        remove = int(request.form[var])
                    except:
                        remove = 0
                    if remove > quantity:
                        error = True
                        text = "Es können max. " + str(quantity) + " Stück entnommen werden"
                        if index == 1:
                            comment1 = text
                            style1 = "background-color: Crimson"
                        if index == 2:
                            comment2 = text
                            style2 = "background-color: Crimson"
                        if index == 3:
                            comment3 = text
                            style3 = "background-color: Crimson"
                        if index == 4:
                            comment4 = text
                            style4 = "background-color: Crimson"
                        if index == 5:
                            comment5 = text
                            style5 = "background-color: Crimson"
        cursor.close()
    
    if not error:
        try:
            quantity = int(quantity1)
            if number1 != "" and quantity > 0:
                movementSave(number1, quantity, notice1, "remove")
                comment1 = "Entnahme gebucht"
        except:
            pass
        try:
            quantity = int(quantity2)
            if number2 != "" and quantity > 0:
                movementSave(number2, quantity, notice2, "remove")
                comment2 = "Entnahme gebucht"
        except:
            pass
        try:
            quantity = int(quantity3)
            if number3 != "" and quantity > 0:
                movementSave(number3, quantity, notice3, "remove")
                comment3 = "Entnahme gebucht"
        except:
            pass
        try:
            quantity = int(quantity4)
            if number4 != "" and quantity > 0:
                movementSave(number4, quantity, notice4, "remove")
                comment4 = "Entnahme gebucht"
        except:
            pass
        try:
            quantity = int(quantity5)
            if number5 != "" and quantity > 0:
                movementSave(number5, quantity, notice5, "remove")
                comment5 = "Entnahme gebucht"
        except:
            pass
    return render_template("removemany.html", number1=number1, quantity1=quantity1, notice1=notice1, number2=number2, quantity2=quantity2, notice2=notice2, number3=number3, quantity3=quantity3, notice3=notice3, number4=number4, quantity4=quantity4, notice4=notice4, number5=number5, quantity5=quantity5, notice5=notice5, comment1=comment1, comment2=comment2, comment3=comment3, comment4=comment4, comment5=comment5, style1=style1, style2=style2, style3=style3, style4=style4, style5=style5, description1=description1, description2=description2, description3=description3, description4=description4, description5=description5, movement=session["movement"], admin=session["admin"], info="", warning="")

# store many
@app.route("/storemany", methods=["GET", "POST"])
def storeMany():
    logging.info("storeMany")
    number1 = ""
    number2 = ""
    number3 = ""
    number4 = ""
    number5 = ""
    quantity1 = ""
    quantity2 = ""
    quantity3 = ""
    quantity4 = ""
    quantity5 = ""
    comment1 = ""
    comment2 = ""
    comment3 = ""
    comment4 = ""
    comment5 = ""
    description1 = ""
    description2 = ""
    description3 = ""
    description4 = ""
    description5 = ""
    style1 = ""
    style2 = ""
    style3 = ""
    style4 = ""
    style5 = ""
    
    try:
        number1 = request.form["number1"]
    except:
        pass
    try:
        number2 = request.form["number2"]
    except:
        pass
    try:
        number3 = request.form["number3"]
    except:
        pass
    try:
        number4 = request.form["number4"]
    except:
        pass
    try:
        number5 = request.form["number5"]
    except:
        pass
    try:
        quantity1 = request.form["quantity1"]
    except:
        pass    
    try:
        quantity2 = request.form["quantity2"]
    except:
        pass    
    try:
        quantity3 = request.form["quantity3"]
    except:
        pass    
    try:
        quantity4 = request.form["quantity4"]
    except:
        pass    
    try:
        quantity5 = request.form["quantity5"]
    except:
        pass    

    action = ""
    try:
        value = request.form["store"]
        action = "store"
    except:
        pass
    
    error = False
    
    if action == "store":
        cursor = mysql.connection.cursor()
        for index in range(1,6):
            var = "number" + str(index)
            try:
                number = request.form[var]
            except:
                number = ""
            if number != "":
                values = {}
                values["number"] = number
                cursor.execute("SELECT quantity, removedquantity, description FROM item WHERE number = %(number)s", values)
                result = cursor.fetchall()
                exists = False
                for line in result:
                    exists = True
                    if index == 1:
                        description1 = line[2]
                    if index == 2:
                        description2 = line[2]
                    if index == 3:
                        description3 = line[2]
                    if index == 4:
                        description4 = line[2]
                    if index == 5:
                        description5 = line[2]
                if not exists:
                    error = True
                    text = "Inventar nicht gefunden"
                    if index == 1:
                        comment1 = text
                        style1 = "background-color: Crimson"
                    if index == 2:
                        comment2 = text
                        style2 = "background-color: Crimson"
                    if index == 3:
                        comment3 = text
                        style3 = "background-color: Crimson"
                    if index == 4:
                        comment4 = text
                        style4 = "background-color: Crimson"
                    if index == 5:
                        comment5 = text
                        style5 = "background-color: Crimson"
                else:
                    quantity = line[1]
                    var = "quantity" + str(index)
                    try:
                        store = int(request.form[var])
                    except:
                        store = 0
                    if store > quantity:
                        error = True
                        text = "Es können max. " + str(quantity) + " Stück eingelagert werden"
                        if index == 1:
                            comment1 = text
                            style1 = "background-color: Crimson"
                        if index == 2:
                            comment2 = text
                            style2 = "background-color: Crimson"
                        if index == 3:
                            comment3 = text
                            style3 = "background-color: Crimson"
                        if index == 4:
                            comment4 = text
                            style4 = "background-color: Crimson"
                        if index == 5:
                            comment5 = text
                            style5 = "background-color: Crimson"
        cursor.close()
    
    if not error:
        try:
            quantity = int(quantity1)
            if number1 != "" and quantity > 0:
                movementSave(number1, quantity, "", "store")
                comment1 = "Zugang gebucht"
        except:
            pass
        try:
            quantity = int(quantity2)
            if number2 != "" and quantity > 0:
                movementSave(number2, quantity, "", "store")
                comment2 = "Zugang gebucht"
        except:
            pass
        try:
            quantity = int(quantity3)
            if number3 != "" and quantity > 0:
                movementSave(number3, quantity, "", "store")
                comment3 = "Zugang gebucht"
        except:
            pass
        try:
            quantity = int(quantity4)
            if number4 != "" and quantity > 0:
                movementSave(number4, quantity, "", "store")
                comment4 = "Zugang gebucht"
        except:
            pass
        try:
            quantity = int(quantity5)
            if number5 != "" and quantity > 0:
                movementSave(number5, quantity, "", "store")
                comment5 = "Zugang gebucht"
        except:
            pass
    return render_template("storemany.html", number1=number1, quantity1=quantity1, number2=number2, quantity2=quantity2, number3=number3, quantity3=quantity3, number4=number4, quantity4=quantity4, number5=number5, quantity5=quantity5, comment1=comment1, comment2=comment2, comment3=comment3, comment4=comment4, comment5=comment5, style1=style1, style2=style2, style3=style3, style4=style4, style5=style5, description1=description1, description2=description2, description3=description3, description4=description4, description5=description5, movement=session["movement"], admin=session["admin"], info="", warning="")

# save new inventory
@app.route("/inventorysavenew", methods=["POST"])
def inventorysavenew():
    logging.info("inventorysavenew")
    checkSession()
    checkBooking()
    values = []
 
    values.append(request.form["description"])
    items = "description, "
    placeholder = "%s, "
    
    values.append(request.form["serialnumber"])
    items = items + "serialnumber, "
    placeholder = placeholder + "%s, "
    
    values.append(request.form["keynumber"])
    items = items + "keynumber, "
    placeholder = placeholder + "%s, "

    try:
        values.append(getStatusNumber(request.form["status"]))
        items = items + "status, "
        placeholder = placeholder + "%s, "
    except Exception as error:
        pass
    
    try:
        values.append(getCategoryNumber(request.form["category"]))
        items = items + "category, "
        placeholder = placeholder + "%s, "
    except Exception as error:
        pass
    
    try:
        values.append(getDetailNumber(request.form["detail"]))
        items = items + "detail, "
        placeholder = placeholder + "%s, "
    except Exception as error:
        pass
    
    try:
        values.append(getLocationNumber(request.form["location"]))
        items = items + "location, "
        placeholder = placeholder + "%s, "
    except Exception as error:
        pass
    
    values.append(request.form["place"])
    items = items + "place, "
    placeholder = placeholder + "%s, "
    
    values.append(request.form["invoice"])
    items = items + "invoice, "
    placeholder = placeholder + "%s, "
    
    pricet = request.form["price"]
    if pricet != "":
        price = float(request.form["price"])
    else:
        price = 0
    values.append(price)
    items = items + "price, "
    placeholder = placeholder + "%s, "
    
    quantity = request.form["quantity"]
    if quantity == "0" or quantity == "":
        quantity = "1"
    values.append(quantity)
    items = items + "quantity, "
    placeholder = placeholder + "%s, "
    
    incoming = request.form["incoming"]
    if incoming != "":
        values.append(request.form["incoming"])
        items = items + "incoming, "
        placeholder = placeholder + "%s, "
    
    lastinspection = request.form["lastinspection"]
    if lastinspection != "":
        values.append(lastinspection)
        items = items + "lastinspection, "
        placeholder = placeholder + "%s, "
        
    values.append(0)
    items = items + "removedquantity, "
    placeholder = placeholder + "%s, "
            
    lastelectricaltest = request.form["lastelectricaltest"]
    if lastelectricaltest != "":
        values.append(lastelectricaltest)
        items = items + "lastelectricaltest, "
        placeholder = placeholder + "%s, "

    outgoing = request.form["outgoing"]
    if outgoing != "":
        values.append(request.form["outgoing"])
        items = items + "outgoing, "
        placeholder = placeholder + "%s, "
    
    currentdate = date.today()
    datestring = currentdate.strftime("%Y-%m-%d")
    values.append(datestring)
    items = items + "changed, "
    placeholder = placeholder + "%s, "
    
    values.append(getUser())
    items = items + "user, "
    placeholder = placeholder + "%s, "
    
    # attachment
    values.append("")
    items = items + "attachment, "
    placeholder = placeholder + "%s, "
        
    values.append(request.form["notice"])
    items = items + "notice, "
    placeholder = placeholder + "%s, "
    
    values.append(0)
    items = items + "updatecounter, "
    placeholder = placeholder + "%s, "

    number = getNextItemNumber()
    values.append(number)
    items = items + "number"
    placeholder = placeholder + "%s"
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("INSERT INTO item (" + items + ") VALUES (" + placeholder + ")", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect(url_for("add", info=number))

def getNextNumber(numberlist, number):
    numbers = numberlist.split(",")
    index = 0
    for numberl in numbers:
        if numberl == number:
            if index + 1 >= len(numbers):
                number = ""
                break
            else:
                number = numbers[index + 1]
                break
        index = index + 1
    return number

# save inventory
@app.route("/inventorysave", methods=["POST"])
def inventorysave():
    logging.info("inventorysave")
    checkSession()
    checkBooking()
    number = request.form["number"]
    numberlist = request.form["numberlist"]
    action = ""
    
    try:
        value = request.form["save"]
        action = "save"
    except:
        pass
    try:
        value = request.form["attachmentadd"]
        action = "attachmentadd"
    except:
        pass
    try:
        value = request.form["attachmentview"]
        action = "attachmentview"
    except:
        pass
    try:
        value = request.form["attachmentdelete"]
        action = "attachmentdelete"
    except:
        pass
    
    if action == "attachmentadd":
        return render_template("attachment.html", number=value, admin=session["admin"], info="", warning="")
        
    if action == "attachmentdelete":
        number, description, serialnumber, keynumber, categorydb, detaildb, statusdb, locationdb, place, invoice, price, quantity, lastinspection, incoming, outgoing, changed, user, updatecounter, statuslist, categorylist, detaillist, locationlist, attachment, notice, lastelectricaltest, removedquantity = getItem(number)
        attachment = ""
        return render_template("inventoryedit.html", number=number, description=description, serialnumber=serialnumber, keynumber=keynumber, categorydb=categorydb, categorylist=categorylist, detaildb=detaildb, detaillist=detaillist, statusdb=statusdb, statuslist=statuslist, locationdb=locationdb, locationlist=locationlist, place=place, invoice=invoice, price=price, quantity=quantity, lastinspection=lastinspection, lastelectricaltest=lastelectricaltest, incoming=incoming, outgoing=outgoing, changed=changed, user=user, updatecounter=updatecounter, numberlist=numberlist, attachment=attachment, notice=notice, admin=session["admin"], info="", warning="")

    if action == "attachmentview":
        filename = AttachmentPath + request.form["attachment"]
        return send_file(filename, as_attachment=True)

    # next
    try:
        value = request.form["next"]
        number = getNextNumber(numberlist, number)
        if number != "":
            number, description, serialnumber, keynumber, categorydb, detaildb, statusdb, locationdb, place, invoice, price, quantity, lastinspection, incoming, outgoing, changed, user, updatecounter, statuslist, categorylist, detaillist, locationlist, attachment, notice, lastelectricaltest, removedquantity = getItem(number)
            return render_template("inventoryedit.html", number=number, description=description, serialnumber=serialnumber, keynumber=keynumber, categorydb=categorydb, categorylist=categorylist, detaildb=detaildb, detaillist=detaillist, statusdb=statusdb, statuslist=statuslist, locationdb=locationdb, locationlist=locationlist, place=place, invoice=invoice, price=price, quantity=quantity, lastinspection=lastinspection, lastelectricaltest=lastelectricaltest, incoming=incoming, outgoing=outgoing, changed=changed, user=user, updatecounter=updatecounter, numberlist=numberlist, attachment=attachment, notice=notice, admin=session["admin"], info="", warning="")
        else:
            return redirect("/search")
    except Exception as error:
        pass

    # history
    try:
        value = request.form["history"]
        description, historylist = history(value)
        return render_template("history.html", number=value, description=description, historylist=historylist, admin=session["admin"], info="", warning="")
    except Exception as error:
        pass
    
    # get update counter 
    updatecounter = int(request.form["updatecounter"])
    valuesselect = {}
    valuesselect["number"] = number
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT updatecounter FROM item WHERE number= %(number)s", valuesselect)
    result = cursor.fetchall()
    valueshistory = []
    for line in result:
        updatecounterdb = int(line[0])
    if updatecounterdb != updatecounter:
        warning = "Inventar " + number + " wurde zwischenzeitlich geändert. Speichern nicht möglich"
        return render_template("inventoryupdateerror.html", admin=session["admin"], info="", warning=warning)
        
    values = []
    values.append(request.form["description"])
    items = "description = %s, "
    
    values.append(request.form["serialnumber"])
    items = items + "serialnumber = %s, "
    
    values.append(request.form["keynumber"])
    items = items + "keynumber = %s, "
    
    try:
        values.append(getStatusNumber(request.form["status"]))
        items = items + "status = %s, "
    except Exception as error:
        pass
    
    try:
        values.append(getCategoryNumber(request.form["category"]))
        items = items + "category = %s, "
    except Exception as error:
        pass
    
    try:
        values.append(getDetailNumber(request.form["detail"]))
        items = items + "detail = %s, "
    except Exception as error:
        pass
    
    try:
        values.append(getLocationNumber(request.form["location"]))
        items = items + "location = %s, "
    except Exception as error:
        pass
    
    values.append(request.form["place"])
    items = items + "place = %s, "
    
    values.append(request.form["invoice"])
    items = items + "invoice = %s, "
    
    pricet = request.form["price"]
    if pricet != "":
        price = float(request.form["price"])
    else:
        price = 0
    values.append(price)
    items = items + "price = %s, "
    
    quantity = request.form["quantity"]
    if quantity == "" or quantity == "1":
        quantity = "1"
    values.append(quantity)
    items = items + "quantity = %s, "
    
    lastinspection = request.form["lastinspection"]
    if lastinspection != "":
        values.append(request.form["lastinspection"])
        items = items + "lastinspection = %s, "
        
    lastelectricaltest = request.form["lastelectricaltest"]
    if lastelectricaltest != "":
        values.append(request.form["lastelectricaltest"])
        items = items + "lastelectricaltest = %s, "
    
    incoming = request.form["incoming"]
    if incoming != "":
        values.append(request.form["incoming"])
        items = items + "incoming = %s, "
    
    outgoing = request.form["outgoing"]
    if outgoing != "":
        values.append(request.form["outgoing"])
        items = items + "outgoing = %s, "
    
    currentdate = date.today()
    datestring = currentdate.strftime("%Y-%m-%d")
    values.append(datestring)
    items = items + "changed = %s, "
    
    values.append(getUser())
    items = items + "user = %s, "
    
    values.append(request.form["attachment"])
    items = items + "attachment = %s, "
        
    values.append(request.form["notice"])
    items = items + "notice = %s, "
        
    updatecounter = updatecounter + 1
    values.append(updatecounter)
    items = items + "updatecounter = %s"
    
    values.append(number)
    # insert history
    cursor = mysql.connection.cursor()
    cursor.execute("LOCK TABLES item WRITE, history WRITE");
    writehistory(number, cursor)
    try:
        cursor.execute("UPDATE item SET " + items + " WHERE number = %s", values)
        cursor.execute("UNLOCK TABLES");
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))

    # edit next item
    if numberlist != "":
        number = getNextNumber(numberlist, number)
        if number != "":
            number, description, serialnumber, keynumber, categorydb, detaildb, statusdb, locationdb, place, invoice, price, quantity, lastinspection, incoming, outgoing, changed, user, updatecounter, statuslist, categorylist, detaillist, locationlist, attachment, notice, lastelectricaltest, removedquantity = getItem(number)
            return render_template("inventoryedit.html", number=number, description=description, serialnumber=serialnumber, keynumber=keynumber, categorydb=categorydb, categorylist=categorylist, detaildb=detaildb, detaillist=detaillist, statusdb=statusdb, statuslist=statuslist, locationdb=locationdb, locationlist=locationlist, place=place, invoice=invoice, price=price, quantity=quantity, lastinspection=lastinspection, lastelectricaltest=lastelectricaltest, incoming=incoming, outgoing=outgoing, changed=changed, user=user, updatecounter=updatecounter, numberlist=numberlist, attachment=attachment, notice=notice, admin=session["admin"], info="", warning="")
    return redirect("/search")

# delete inventory
@app.route("/inventorydelete", methods=["POST"])
def inventorydelete():
    logging.info("inventorydelete")
    checkSession()
    checkBooking()
    number = request.form["number"]
    # history?
    try:
        value = request.form["history"]
        description, historylist = history(value)
        return render_template("history.html", number=value, description=description, historylist=historylist, admin=session["admin"], info="", warning="")
    except:
        pass
    
    # insert history
    cursor = mysql.connection.cursor()
    cursor.execute("LOCK TABLES item WRITE, history WRITE");
    writehistory(number, cursor)
    values = []
    values.append(number)
    try:
        cursor.execute("DELETE FROM item WHERE number = %s", values)
        cursor.execute("UNLOCK TABLES");
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect ("/search")
 
# -------------------------------------------------------------    
# history
def history(number):
    logging.info("history")
    values = {}
    values["number"] = number
    description = ""
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT description, changed, user FROM history WHERE itemnumber = %(number)s ORDER BY changed, number", values)
        result = cursor.fetchall()
        historylist = []
        for line in result:
            description = line[0]
            history = []
            if line[1] != None:
                datedb = line[1]
                localedate = datedb.strftime("%d.%m.%Y")
                history.append(localedate)
            else:
                history.append("")
            history.append(line[2])
            historylist.append(history)
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return description, historylist

@app.route("/historyview", methods=["POST"])
def historyview():
    logging.info("historyview")
    checkSession()
    number = request.form["number"]
    values = {}
    values["number"] = number
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT itemnumber, description, serialnumber, keynumber, category, detail, status, location, place, price, quantity, invoice, lastinspection, incoming, outgoing, changed, user, attachment, notice, lastelectricaltest FROM history WHERE itemnumber= %(number)s", values)
        result = cursor.fetchall()
        for line in result:
            if (line[0] is None):
                itemnumber = "0"
            else:
                itemnumber = str(line[0])
            if (line[1] is None):
                description = ""
            else:
                description = line[1]
            if (line[2] is None):
                serialnumber = ""
            else:
                serialnumber = line[2]
            if (line[3] is None):
                keynumber = ""
            else:
                keynumber = line[3]
            if (line[4] is None):
                category = ""
                categorydb = ""
            else:
                category = getCategoryDescription(line[4])
                categorydb = category
            if (line[5] is None):
                detail = ""
                detaildb = ""
            else:
                detail = getDetailDescription(line[5])
                detaildb = detail
            if (line[6] is None):
                status = ""
                statusdb = ""
            else:
                status = getStatusDescription(line[6])
                statusdb = status
            if (line[7] is None):
                location = ""
                locationdb = ""
            else:
                location = getLocationDescription(line[7])
                locationdb = location
            if (line[8] is None):
                place = ""
            else:
                place = line[8]
            if (line[9] is None):
                price = ""
            else:
                price = line[9]
            if (line[10] is None):
                quantity = ""
            else:
                quantity = line[10]
            if (line[11] is None):
                invoice = ""
            else:
                invoice = line[11]
            if (line[12] is None):
                lastinspection = ""
            else:
                lastinspection = line[12]
            if (line[13] is None):
                incoming = ""
            else:
                incoming = line[13]
            if (line[14] is None):
                outgoing = ""
            else:
                outgoing = line[14]
            if (line[15] is None):
                changed = ""
            else:
                changed = line[15]
            if (line[16] is None):
                user = ""
            else:
                user = line[16]
            if (line[17] is None):
                attachment = ""
            else:
                attachment = line[17]
            if (line[18] is None):
                notice = ""
            else:
                notice = line[18]
            if (line[19] is None):
                lastelectricaltest = ""
            else:
                lastelectricaltest = line[19]
        cursor.close()
        statuslist = getStatusList()
        categorylist = getCategoryList()
        detaillist = getDetailList()
        locationlist = getLocationList()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return render_template("historyview.html", number=itemnumber, description=description, serialnumber=serialnumber, keynumber=keynumber, category=category, detail=detail, status=status, location=location, place=place, invoice=invoice, price=price, quantity=quantity, lastinspection=lastinspection, lastelectricaltest=lastelectricaltest, incoming=incoming, outgoing=outgoing, changed=changed, user=user, attachment=attachment, notice=notice, admin=session["admin"], info="", warning="")

@app.route("/historyattachment", methods=["POST"])
def historyattachment():
    logging.info("historyattachment")
    checkSession()
    checkBooking()
    
    try:
        value = request.form["attachmentview"]
        filename = AttachmentArchivPath + value
        return send_file(filename, as_attachment=True)
    except Exception as error:
        logging.error(str(error))
    return redirect("/search")

# -------------------------------------------------------------    
# export
@app.route("/export")
def export():
    logging.info("export")
    checkSession()
    checkBooking()
    statuslist = getStatusList()
    categorylist = getCategoryList()
    locationlist = getLocationList()
    return render_template("export.html", admin=session["admin"], info="", warning="")

@app.route("/exportfile", methods=["POST"])
def exportfile():
    logging.info("exportfile")
    checkSession()
    temp = tempfile.NamedTemporaryFile()
    tempfilename = temp.name + ".csv" 
    file = open(tempfilename, "wb")
    textline = "Nummer;Beschreibung;Seriennummer;Schlüsselnummer;Kategorie;Detail;Status;Standort;Platz;Rechnung;Preis;Anzahl;letzte Inventur;Zugangsdatum;Abgangsdatum\n"
    textline = textline.encode("utf-16")
    file.write(textline)
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number, description, serialnumber, keynumber, category, detail, status, location, place, invoice, price, quantity, lastinspection, incoming, outgoing FROM item ORDER BY number")
        result = cursor.fetchall()
        for line in result:
            if (line[0] is None):
                number = "0"
            else:
                number = str(line[0])
            if (line[1] is None):
                description = ""
            else:
                description = line[1]
                description = description.replace("\t", " ")
                description = description.replace("\r", " ")
                description = description.replace("\n", " ")
                description = description.strip()
            if (line[2] is None):
                serialnumber = ""
            else:
                serialnumber = line[2]
            if (line[3] is None):
                keynumber = ""
            else:
                keynumber = line[3]
            if (line[4] is None):
                category = ""
            else:
                category = getCategoryDescription(line[4])
            if (line[5] is None):
                detail = ""
            else:
                detail = getDetailDescription(line[5])
                if (line[6] is None):
                    status = ""
                else:
                    status = getStatusDescription(line[6])
            if (line[7] is None):
                location = ""
            else:
                location = getLocationDescription(line[7])
            if (line[8] is None):
                place = ""
            else:
                place = line[8]
            if (line[9] is None):
                invoice = ""
            else:
                invoice = line[9]
            if (line[10] is None):
                price = "0"
            else:
                price = str(line[10])
            if (line[11] is None):
                quantity = ""
            else:
                quantity = str(line[11])
            if (line[12] is None):
                lastinspection = ""
            else:
                lastinspection = str(line[12])
            if (line[13] is None):
                incoming = ""
            else:
                incoming = str(line[13])
            if (line[14] is None):
                outgoing = ""
            else:
                outgoing = str(line[14])
            
            textline = number + ";" + description + ";" + serialnumber + ";" + keynumber + ";" + category + ";" + detail + ";" + status + ";" + location + ";" + place + ";" + invoice + ";" + price + ";" + quantity + ";" + lastinspection + ";" + incoming + ";" + outgoing + "\n"
            textline = textline.encode("utf-16")
            file.write(textline)
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    file.close()
    return send_file(tempfilename, as_attachment=True)

# -------------------------------------------------------------    
# upload attachment
@app.route("/uploadattachment", methods=["POST"])
def uploadattachment():
    logging.info("uploadattachment")
    checkSession()
    checkBooking()
    warning = ""
    files = request.files.getlist("file")
    for file in files:
        if file.filename != "":
            filename =  secure_filename(file.filename)
            file.save(AttachmentPath + filename)
    number = request.form["number"]
    number, description, serialnumber, keynumber, categorydb, detaildb, statusdb, locationdb, place, invoice, price, quantity, lastinspection, incoming, outgoing, changed, user, updatecounter, statuslist, categorylist, detaillist, locationlist, attachment , notice, lastelectricaltest, removedquantity = getItem(number)
    return render_template("inventoryedit.html", number=number, description=description, serialnumber=serialnumber, keynumber=keynumber, categorydb=categorydb, categorylist=categorylist, detaildb=detaildb, detaillist=detaillist, statusdb=statusdb, statuslist=statuslist, locationdb=locationdb, locationlist=locationlist, place=place, invoice=invoice, price=price, quantity=quantity, lastinspection=lastinspection, lastelectricaltest=lastelectricaltest, incoming=incoming, outgoing=outgoing, changed=changed, user=user, updatecounter=updatecounter, numberlist="", attachment=filename, admin=session["admin"], info="", warning="")

# -------------------------------------------------------------    
# import
@app.route("/import")
def importfile():
    logging.info("importfile")
    checkSession()
    checkBooking()
    return render_template("upload.html", admin=session["admin"], info="Nur CSV-Datei", warning="")

# upload file
@app.route("/uploadfile", methods=["POST"])
def uploadfile():
    logging.info("uploadfile")
    checkSession()
    checkBooking()
    warning = ""
    filename = ""
    importfile = request.files.getlist("file")
    for file in importfile:
        if file.filename != "":
            extension = file.filename.rsplit(".")
            if extension[1].lower() == "csv": 
                filename = app.config["UPLOAD_FOLDER"] + "import_" + secure_filename(file.filename)
                file.save(filename)
            else:
                return render_template("import.html", info="Nur CSV-Datei möglich", warning="", file=file, admin=session["admin"])
    
    if filename == "":
        return render_template("upload.html", admin=session["admin"], info="Nur CSV-Datei", warning="Keine Datei ausgewählt")
        
    pos = 0
    file = open(filename, "r", encoding="utf-8")
    lines = file.readlines()
    warning = ""
    
    for line in lines:
        line = line.strip()
        pos = pos + 1
        # first line is header
        #Bezeichnung;Seriennummer;Schluesselnummer;Kategorie;Detail;Status;Standort;Platz;Rechnung;Wert;Anzahl;Zugangsdatum;Abgangsdatum;letzte Inventur
        if pos != 1:
            valueline = line.split(";")
            values = []
            valuepos = 0
            for value in valueline:
                if valuepos > 13:
                    break
                values.append(str(value))
                valuepos = valuepos + 1

            category = values[3]
            detail = values[4]
            status = values[5]
            location = values[6]
            # Quantity
            if values[10] == "":
                values[10] = "1";
            incoming = values[11]
            outgoing = values[12]
            lastinspection = values[13]
            # check input
            warning = ""
            number = getCategoryNumber(category)
            if number == 0:
                warning = "Zeile: " + str(pos) + ": Kategorie " + category + " nicht gefunden"
                break
            values[3] = number
            number = getDetailNumber(detail)
            if number == 0:
                warning = "Zeile: " + str(pos) + ": Detail " + detail + " nicht gefunden"
                break
            values[4] = number
            number = getStatusNumber(status)
            if number == 0:
                warning = "Zeile: " + str(pos) + ": Status " + status + " nicht gefunden"
                break
            values[5] = number
            number = getLocationNumber(location)
            if number == 0:
                warning = "Zeile: " + str(pos) + ": Standort " + location + " nicht gefunden"
                break
            values[6] = number
        
            if incoming == "":
                warning = "Zeile: " + str(pos) + ": Zugangsdatum fehlt"
                break
            try:
                localedate = datetime.strptime(incoming, "%d.%m.%Y")
                values[11] = localedate.date()
            except Exception as error:
                warning = "Zeile: " + str(pos) + ": Zugangsdatum " + incoming + " falsch"
                break
            
            if outgoing == "":
                values[12] = MaxDate
            else:
                try:
                    localedate = datetime.strptime(outgoing, "%d.%m.%Y")
                    values[12] = localedate.date()
                except Exception as error:
                    warning = "Zeile: " + str(pos) + ": Abgangsdatum " + outgoing + " falsch"
                    break
            
            if lastinspection == "":
                values[13] = values[11]
                
            try:
                localedate = datetime.strptime(lastinspection, "%d.%m.%Y")
                values[13] = localedate.date()
            except Exception as error:
                warning = "Zeile: " + str(pos) + ": Letzte Prüfung " + lastinspection + " falsch"
                break
            
            # update counter
            values.append(0)
            number = getNextItemNumber()
            startnumber = number
            values.append(number)
            try:
                cursor = mysql.connection.cursor()
                cursor.execute("INSERT INTO item (description, serialnumber, keynumber, category, detail, status, location, place, invoice, price, quantity, incoming, outgoing, lastinspection, updatecounter, number) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)", values)
                cursor.close()
            except Exception as error:
                mysql.connection.rollback()
                logging.error(str(error))
                return jsonify("error: " + str(error))
            mysql.connection.commit()
    
    file.close()
    if warning != "":
        return render_template("upload.html", info="Nur CSV-Datei möglich", warning=warning, admin=session["admin"])

    pos = pos - 1
    if pos > 0:
        info =  "Ab Inventarnummer " + str(startnumber) + ", " + str(pos) + " Inventare hinzugefügt"
    else:
        info = "Kein Invetar hinzugefügt"
    return render_template("upload.html", info=info, warning="", admin=session["admin"])
        
# -------------------------------------------------------------    
# location
@app.route("/location")
def location():
    logging.info("location")
    checkSession()
    checkAdmin()
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT number, description FROM location")
    result = cursor.fetchall()
    locationlist = []
    for line in result:
        location = []
        location.append(line[0])
        location.append(line[1])
        locationlist.append(location)
    cursor.close()
    return render_template("location.html", locationlist=locationlist, admin=session["admin"], info="", warning="")

# edit location
@app.route("/locationedit", methods=["POST"])
def locationedit():
    logging.info("locationedit")
    checkSession()
    checkAdmin()
    
    try:
        value = request.form["edit"]
        action = "edit"
    except:
        try:
            value = request.form["delete"]
            action = "delete"
        except: 
            value = request.form["new"]
            action = "new"
    
    if action == "new":
        return render_template("locationnew.html", admin=session["admin"], info="", warning="")

    values = {}
    values["number"] = value
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT number, description FROM location WHERE number= %(number)s", values)
    result = cursor.fetchall()
    for line in result:
        number = line[0]
        description = line[1]
    login = False
    cursor.close()
    if action == "delete":
        return render_template("locationdelete.html", number=number, description=description, admin=session["admin"], info="", warning="")
    return render_template("locationedit.html", number=number, description=description, admin=session["admin"], info="", warning="")

# new location
@app.route("/locationsavenew", methods=["POST"])
def locationsavenew():
    logging.info("locationsavenew")
    checkSession()
    checkAdmin()
    values = []
    values.append(request.form["description"])
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("INSERT INTO location (description) VALUES (%s)", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect("/location")

# save edit location
@app.route("/locationsave", methods=["POST"])
def locationsave():
    logging.info("locationsave")
    checkSession()
    checkAdmin()
    values = []
    values.append(request.form["description"])
    values.append(request.form["number"])
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("UPDATE location SET description = %s WHERE number = %s", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect ("/location")

# delete location
@app.route("/locationdelete", methods=["POST"])
def locationdelete():
    logging.info("locationdelete")
    checkSession()
    checkAdmin()

    select = {}
    select["number"] = request.form["number"]
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number FROM item WHERE location= %(number)s", select)
        result = cursor.fetchall()
        if result:
            cursor.close()
            return render_template("locationdelete.html", admin=session["admin"], number=number, description=description, info="Inventar mit diesem Standort vorhanden. Löschen nicht möglich", warning="")
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    
    values = []
    values.append(request.form["number"])
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("DELETE FROM location WHERE number = %s", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect ("/location")

# -------------------------------------------------------------    
# category
@app.route("/category")
def category():
    logging.info("category")
    checkSession()
    checkAdmin()
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT number, description FROM category")
    result = cursor.fetchall()
    categorylist = []
    for line in result:
        category = []
        category.append(line[0])
        category.append(line[1])
        categorylist.append(category)
    cursor.close()
    return render_template("category.html", categorylist=categorylist, admin=session["admin"], info="", warning="")

# edit category
@app.route("/categoryedit", methods=["POST"])
def categoryedit():
    logging.info("categoryedit")
    checkSession()
    checkAdmin()
    
    try:
        value = request.form["edit"]
        action = "edit"
    except:
        try:
            value = request.form["delete"]
            action = "delete"
        except: 
            value = request.form["new"]
            action = "new"
    
    if action == "new":
        return render_template("categorynew.html", admin=session["admin"], info="", warning="")

    values = {}
    values["number"] = value
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT number, description FROM category WHERE number= %(number)s", values)
    result = cursor.fetchall()
    for line in result:
        number = line[0]
        description = line[1]
    login = False
    cursor.close()
    if action == "delete":
        return render_template("categorydelete.html", number=number, description=description, admin=session["admin"], info="", warning="")
    return render_template("categoryedit.html", number=number, description=description, admin=session["admin"], info="", warning="")

# new category
@app.route("/categorysavenew", methods=["POST"])
def categorysavenew():
    logging.info("categorysavenew")
    checkSession()
    checkAdmin()
    values = []
    values.append(request.form["description"])
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("INSERT INTO category (description) VALUES (%s)", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect("/category")

# save edit category
@app.route("/categorysave", methods=["POST"])
def categorysave():
    logging.info("categorysave")
    checkSession()
    checkAdmin()
    values = []
    values.append(request.form["description"])
    values.append(request.form["number"])
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("UPDATE category SET description = %s WHERE number = %s", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect ("/category")

# delete category
@app.route("/categorydelete", methods=["POST"])
def categorydelete():
    logging.info("categorydelete")
    checkSession()
    checkAdmin()

    select = {}
    select["number"] = request.form["number"]
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number FROM item WHERE category= %(number)s", select)
        result = cursor.fetchall()
        cursor.close()
        if result:
            return render_template("categorydelete.html", admin=session["admin"], number=number, description=description, info="Inventar mit dieser Kategorie vorhanden. Löschen nicht möglich", warning="")
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    
    values = []
    values.append(request.form["number"])
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("DELETE FROM category WHERE number = %s", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect ("/category")

# -------------------------------------------------------------    
# detail
@app.route("/detail")
def detail():
    logging.info("detail")
    checkSession()
    checkAdmin()
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT number, description FROM detail")
    result = cursor.fetchall()
    detaillist = []
    for line in result:
        detail = []
        detail.append(line[0])
        detail.append(line[1])
        detaillist.append(detail)
    cursor.close()
    return render_template("detail.html", detaillist=detaillist, admin=session["admin"], info="", warning="")

# edit detail
@app.route("/detailedit", methods=["POST"])
def detailedit():
    logging.info("detailedit")
    checkSession()
    checkAdmin()
    
    try:
        value = request.form["edit"]
        action = "edit"
    except:
        try:
            value = request.form["delete"]
            action = "delete"
        except: 
            value = request.form["new"]
            action = "new"
    
    if action == "new":
        return render_template("detailnew.html", admin=session["admin"], info="", warning="")

    values = {}
    values["number"] = value
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT number, description FROM detail WHERE number= %(number)s", values)
    result = cursor.fetchall()
    for line in result:
        number = line[0]
        description = line[1]
    cursor.close()
    if action == "delete":
        return render_template("detaildelete.html", number=number, description=description, admin=session["admin"], info="", warning="")
    return render_template("detailedit.html", number=number, description=description, admin=session["admin"], info="", warning="")

# new detail
@app.route("/detailsavenew", methods=["POST"])
def detailsavenew():
    logging.info("detailsavenew")
    checkSession()
    checkAdmin()
    values = []
    values.append(request.form["description"])
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("INSERT INTO detail (description) VALUES (%s)", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect("/detail")

# save edit detail
@app.route("/detailsave", methods=["POST"])
def detailsave():
    logging.info("detailsave")
    checkSession()
    checkAdmin()
    values = []
    values.append(request.form["description"])
    values.append(request.form["number"])
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("UPDATE detail SET description = %s WHERE number = %s", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect ("/detail")

# delete detail
@app.route("/detaildelete", methods=["POST"])
def detaildelete():
    logging.info("detaildelete")
    checkSession()
    checkAdmin()
    
    select = {}
    select["number"] = request.form["number"]
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number FROM item WHERE detail= %(number)s", select)
        result = cursor.fetchall()
        cursor.close()
        if result:
            return render_template("detaildelete.html", admin=session["admin"], number=number, description=description, info="Inventar mit diesem Detail vorhanden. Löschen nicht möglich", warning="")
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    
    values = []
    values.append(request.form["number"])
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("DELETE FROM detail WHERE number = %s", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect ("/detail")

# -------------------------------------------------------------    
# status
@app.route("/status")
def status():
    logging.info("status")
    checkSession()
    checkAdmin()
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT number, description FROM status")
    result = cursor.fetchall()
    statuslist = []
    for line in result:
        status = []
        status.append(line[0])
        status.append(line[1])
        statuslist.append(status)
    cursor.close()
    return render_template("status.html", statuslist=statuslist, admin=session["admin"], info="", warning="")

# edit status
@app.route("/statusedit", methods=["POST"])
def statusedit():
    logging.info("statusedit")
    checkSession()
    checkAdmin()
    
    try:
        value = request.form["edit"]
        action = "edit"
    except:
        try:
            value = request.form["delete"]
            action = "delete"
        except: 
            value = request.form["new"]
            action = "new"
    
    if action == "new":
        return render_template("statusnew.html", admin=session["admin"], info="", warning="")

    values = {}
    values["number"] = value
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT number, description FROM status WHERE number= %(number)s", values)
    result = cursor.fetchall()
    for line in result:
        number = line[0]
        description = line[1]
    login = False
    cursor.close()
    if action == "delete":
        return render_template("statusdelete.html", admin=session["admin"], info="", warning="")
    return render_template("statusedit.html", number=number, description=description, admin=session["admin"], info="", warning="")

# new status
@app.route("/statussavenew", methods=["POST"])
def statussavenew():
    logging.info("statussavenew")
    checkSession()
    checkAdmin()
    values = []
    values.append(request.form["description"])
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("INSERT INTO status (description) VALUES (%s)", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect("/status")

# save edit status
@app.route("/statussave", methods=["POST"])
def statussave():
    logging.info("statussave")
    checkSession()
    checkAdmin()
    values = []
    values.append(request.form["description"])
    values.append(request.form["number"])
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("UPDATE status SET description = %s WHERE number = %s", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect ("/status")

# delete status
@app.route("/statusdelete", methods=["POST"])
def statusdelete():
    logging.info("statusdelete")
    checkSession()
    checkAdmin()
    
    select = {}
    select["number"] = request.form["number"]
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number FROM item WHERE status= %(number)s", select)
        result = cursor.fetchall()
        cursor.close()
        if result:
            return render_template("statusdelete.html", admin=session["admin"], number=number, description=description, info="Inventar mit diesem Status vorhanden. Löschen nicht möglich", warning="")
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    
    values = []
    values.append(request.form["number"])
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("DELETE FROM status WHERE number = %s", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect ("/status")

# -------------------------------------------------------------    
# invent
def getBarcodeFiles():
    directories = os.listdir(FTPFilePath)
    filelist = []
    for filename in directories:
        if os.path.isfile(FTPFilePath + filename):
            filevalues = []
            filevalues.append(filename)
            filesize = os.path.getsize(FTPFilePath + filename)
            filevalues.append(filesize)
            timestamp = os.path.getctime(FTPFilePath + filename)
            filetime = datetime.fromtimestamp(timestamp)
            filetime = datetime.strftime(filetime, "%d.%m.%Y %H:%M:%S")
            filevalues.append(filetime)
            if "upload" in filename:
                filevalues.append("U")
            else:
                filevalues.append("")
            filelist.append(filevalues)
    currentdate = date.today()
    return filelist, currentdate

@app.route("/invent")
def invent():
    logging.info("invent")
    checkSession()
    filelist, currentdate = getBarcodeFiles()
    return render_template("invent.html", filelist=filelist, currentdate=currentdate, admin=session["admin"], info="", warning="")

@app.route("/inventexec", methods=["POST"])
def inventexec():
    logging.info("inventexec")
    checkSession()
    checkBooking()
    
    try:
        value = request.form["view"]
        action = "view"
    except:
        pass
    try:
        value = request.form["upload"]
        action = "upload"
    except:
        pass
    try:
        value = request.form["delete"]
        action = "delete"
    except:
        pass
    
    filename = value
    if action == "upload":
        inventlist = listBarcodefile(filename)
        currentdate = date.today()
        return render_template("inventupload.html", filename=filename, inventlist=inventlist, currentdate=currentdate, admin=session["admin"], info="", warning="")        

    if action == "view":
        inventlist = listBarcodefile(filename)
        return render_template("inventview.html", filename=filename, inventlist=inventlist, admin=session["admin"], info="", warning="")
    
    if action == "delete":
        filename = FTPFilePath + value
        os.remove(filename)
    return redirect("/invent")

@app.route("/inventupload", methods=["POST"])
def inventupload():
    logging.info("inventupload")
    checkSession()
    checkBooking()
    filename = request.form["filename"]
    inspectiondate = request.form["inspectiondate"]
    items, located = inventUpdate(FTPFilePath + filename, inspectiondate)
    filelist, currentdate = getBarcodeFiles()
    info = "Datei " + filename + " eingelesen. " + str(located) + " gefunden, " + str(items) + " geändert"
    return render_template("invent.html", filelist=filelist, currentdate=currentdate, admin=session["admin"], info=info, warning="")

def listBarcodefile(filename):
    file = open(FTPFilePath + filename, "r", encoding="utf-8")
    lines = file.readlines()
    inventlist = []
    for line in lines:
        line = line.split(";")
        number = line[0]
        if line[1] == "" or line[1] == "0":
            quantity = 1
        else:
            try:
                quantity = int(line[1])
            except:
                quantity = 1
        result = isItem(number)
        itemquantity = getQuantity(number)
        itemlist = []
        itemlist.append(number)
        itemlist.append(quantity)
        note = ""
        color = ""
        if quantity != itemquantity:
            note = "Menge Invenar: " + str(itemquantity)
            color = "background-color:OrangeRed"
        if not result:
            note = "Inventar nicht gefunden"
            color = "background-color:OrangeRed"
        itemlist.append(note)
        itemlist.append(color)
        inventlist.append(itemlist)
    file.close()
    return inventlist

def inventUpdate(filename, inspectiondate):
    logging.info("inventUpdate")
    file = open(filename, "r", encoding="utf-8")
    cursor = mysql.connection.cursor()
    lines = file.readlines()
    items = 0
    exists = 0
    for line in lines:
        line = line.split(";")
        item = line[0]
        try:
            quantity = int(line[1])
        except:
            quantity = 1
        if quantity == 0:
            quantity = 1
        if isItem(item):
            exists = exists + 1
        values = {}
        values["inspectiondate"] = inspectiondate
        values["number"] = item
        values["quantity"] = quantity
        cursor.execute("UPDATE item SET lastinspection = %(inspectiondate)s, quantity = %(quantity)s WHERE number = %(number)s", values)
        cursor.execute("SELECT ROW_COUNT()")
        result = cursor.fetchall()
        for res in result:
            if res[0] > 0:
                items = items + 1
    mysql.connection.commit()
    filenamenew = filename + "_upload"
    os.rename(filename, filenamenew)
    cursor.close()
    file.close()
    return items, exists

# -------------------------------------------------------------    
# labelsize
@app.route("/labelsizelist")
def labelsizelist():
    logging.info("labelsizelist")
    checkSession()
    checkAdmin()
    
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number, description, pagesize, width, height, marginleft, margintop, defaultlabel FROM label")
        result = cursor.fetchall()
        labelsizelist = []
        for line in result:
            labelsize = []
            labelsize.append(line[0])
            labelsize.append(line[1])
            labelsize.append(line[2])
            labelsize.append(line[3])
            labelsize.append(line[4])
            labelsize.append(line[5])
            labelsize.append(line[6])
            if line[7] == 1:
                labelsize.append("checked")
            else:
                labelsize.append("")
            labelsizelist.append(labelsize)
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return render_template("labelsizelist.html", labelsizelist=labelsizelist, admin=session["admin"], info="", warning="")

# edit labelsize
@app.route("/labelsizeedit", methods=["POST"])
def labelsizeedit():
    logging.info("labelsizeedit")
    checkSession()
    checkAdmin()
    try:
        value = request.form["edit"]
        action = "edit"
    except:
        try:
            value = request.form["delete"]
            action = "delete"
        except: 
            value = request.form["new"]
            action = "new"
    
    if action == "new":
        return render_template("labelsizenew.html", scalewidth="1", scaleheight="1", admin=session["admin"], info="", warning="")

    values = {}
    values["number"] = value
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT number, description, pagesize, width, height, marginlabelright, marginlabelbottom, marginleft, marginright, margintop, marginbottom, line1, line2, scalewidth, scaleheight, defaultlabel FROM label WHERE number= %(number)s", values)
    result = cursor.fetchall()
    line = result[0]
    cursor.close()
    default = ""
    if line[15] == 1:
        default = "checked"
    
    if action == "delete":
        return render_template("labelsizedelete.html", number=line[0], description=line[1], pagesize=line[2], width=line[3], height=line[4], marginlabelright=line[5], marginlabelbottom=line[6], marginleft=line[7], marginright=line[8], margintop=line[9], marginbottom=line[10], line1=line[11], line2=line[12], scalewidth=line[13], scaleheight=line[14], default=default, admin=session["admin"], info="", warning="")
    return render_template("labelsizeedit.html", number=line[0], description=line[1], pagesize=line[2], width=line[3], height=line[4], marginlabelright=line[5], marginlabelbottom=line[6], marginleft=line[7], marginright=line[8], margintop=line[9], marginbottom=line[10], line1=line[11], line2=line[12], scalewidth=line[13], scaleheight=line[14], default=default, admin=session["admin"], info="", warning="")

# labelsize save new
@app.route("/labelsizesavenew", methods=["POST"])
def labelsizesavenew():
    logging.info("labelsizesavenew")
    checkSession()
    checkAdmin()
    try:
        values = []
        values.append(request.form["description"])
        pagesize = request.form["pagesize"]
        pagesize = pagesize.upper()
        values.append(pagesize)
        values.append(request.form["width"])
        values.append(request.form["height"])
        values.append(request.form["marginlabelright"])
        values.append(request.form["marginlabelbottom"])
        values.append(request.form["marginleft"])
        values.append(request.form["marginright"])
        values.append(request.form["margintop"])
        values.append(request.form["marginbottom"])
        values.append(request.form["line1"])
        values.append(request.form["line2"])
        values.append(request.form["scalewidth"])
        values.append(request.form["scaleheight"])
        valuedefault = request.form.get("default", default=False, type=bool)
        if valuedefault:
            values.append(1)
        else:
            values.append(0)
        cursor = mysql.connection.cursor()
        cursor.execute("INSERT INTO label (description, pagesize, width, height, marginlabelright, marginlabelbottom, marginleft, marginright, margintop, marginbottom, line1, line2, scalewidth, scaleheight, defaultlabel) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect("/labelsizelist")

# labelsize save 
@app.route("/labelsizesave", methods=["POST"])
def labelsizesave():
    logging.info("labelsizesave")
    checkSession()
    checkAdmin()
    try:
        values = []
        values.append(request.form["description"])
        pagesize = request.form["pagesize"]
        pagesize = pagesize.upper()
        values.append(pagesize)
        width = float(request.form["width"])
        values.append(width)
        height = float(request.form["height"])
        values.append(height)
        marginlabelright = float(request.form["marginlabelright"])
        values.append(marginlabelright)
        marginlabelbottom = float(request.form["marginlabelbottom"])
        values.append(marginlabelbottom)
        marginleft = float(request.form["marginleft"])
        values.append(marginleft)
        marginright = float(request.form["marginright"])
        values.append(marginright)
        margintop = float(request.form["margintop"])
        values.append(margintop)
        marginbottom = float(request.form["marginbottom"])
        values.append(marginbottom)
        values.append(request.form["line1"])
        values.append(request.form["line2"])
        scalewidth = float(request.form["scalewidth"])
        values.append(scalewidth)
        scaleheight = float(request.form["scaleheight"])
        values.append(scaleheight)
        valuedefault = request.form.get("default", default=False, type=bool)
        if valuedefault:
            values.append(1)
        else:
            values.append(0)
        values.append(request.form["number"])
        cursor = mysql.connection.cursor()
        cursor.execute("UPDATE label SET description = %s, pagesize = %s, width = %s, height = %s, marginlabelright = %s, marginlabelbottom = %s, marginleft = %s, marginright = %s, margintop = %s, marginbottom = %s, line1 = %s, line2 = %s, scalewidth = %s, scaleheight = %s, defaultlabel = %s WHERE number = %s", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect("/labelsizelist")

# label size delete
@app.route("/labelsizedelete", methods=["POST"])
def labelsizedelete():
    logging.info("labeldelete")
    checkSession()
    checkAdmin()
    values = {}
    values["number"] = request.form["number"]
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("DELETE FROM label WHERE number = %(number)s", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect("/labelsizelist")

# -------------------------------------------------------------    
# print label
def replaceName(line, item):
    result = ""
    if line == "[Nummer]":
        result = item[0]
    if line == "[Bezeichnung]":
        result = item[1]
    if line == "[Seriennummer]":
        result = item[2]
    if line == "[Kategoprie]":
        result = getCategoryDescription(item[3])
    if line == "[Detail]":
        result = getDetailDescription(item[4])
    if line == "[Standort]":
        result = getLocationDescription(item[5])
    if line == "[Platz]":
        result = item[6]
    if result == "":
        result = line
    return result

@app.route("/printlabel", methods=["POST"])
def printlabel():
    logging.info("printlabel")
    checkSession()
    values = {}
    values["description"] = request.form["labelsize"]
    # get label size and content
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT pagesize, width, height, marginlabelright, marginlabelbottom, marginleft, marginright, margintop, marginbottom, line1, line2, scalewidth, scaleheight FROM label WHERE description = %(description)s", values)
        result = cursor.fetchall()
        for line in result:
            pagesize = line[0]
            width = line[1]
            height = line[2]
            marginlabelright = line[3]
            marginlabelbottom = line[4]
            marginleft = line[5]
            marginright = line[6]
            margintop = line[7]
            marginbottom = line[8]
            labelline1 = line[9]
            labelline2 = line[10]
            scalewidth = line[11]
            scaleheight = line[12]
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    
    document = label.Label()
    document.setPath(TempPath)
    document.setPage(pagesize)
    document.setSize(width, height)
    document.setMargin(marginlabelright, marginlabelbottom)
    document.setMarginPage(marginleft, marginright, margintop, marginbottom)
    document.setScale(scalewidth, scaleheight)
    row = int(request.form["row"])
    column = int(request.form["column"])
    document.setStart(row, column)
    numberlist = request.form["numberlist"]
    numberlist = numberlist.split(",")
    for number in numberlist:
        values = {}
        values["number"] = number
        try:
            cursor = mysql.connection.cursor()
            cursor.execute("SELECT number, description, serialnumber, category, detail, location, place, quantity FROM item WHERE number= %(number)s", values)
            result = cursor.fetchall()
            for item in result:
                number = item[0]
                line1 = replaceName(labelline1, item)
                line2 = replaceName(labelline2, item)
                quantity = item[7]
                for counter in range(quantity):
                    document.addLabel(line1, line2, number)
            cursor.close()
        except Exception as error:
            mysql.connection.rollback()
            logging.error(str(error))
            return jsonify("error: " + str(error))
    pdffile = document.createPDF()
    return send_file(pdffile, as_attachment=True)

# -------------------------------------------------------------    
# print item list
def printitemlist(numberlist):
    logging.info("printitemlist")
    checkSession()
    checkBooking()
    document = itemlist.Liste()
    document.setPath(TempPath)
    document.setMarginPage(20, 20, 20, 20)
    currentdate = date.today()
    datestring = currentdate.strftime("%d.%m.%Y")
    document.setTitle("Inventurliste vom " + datestring + "                                     Seite: ")
    document.addTab(1)
    document.addTab(8)
    document.addTab(30)
    document.addTab(45)
    document.addTab(55)
    document.addTab(70)
    document.addHeader("Num")
    document.addHeader("Bezeichnung")
    document.addHeader("Seriennummer")
    document.addHeader("Standort")
    document.addHeader("Platz")
    document.addHeader("Status")

    numberlist = numberlist.split(",")
    for number in numberlist:
        values = {}
        values["number"] = number
        try:
            cursor = mysql.connection.cursor()
            cursor.execute("SELECT number, description, serialnumber, location, place, status FROM item WHERE number= %(number)s", values)
            result = cursor.fetchall()
            for item in result:
                number = item[0]
                description = item[1]
                serialnumber = item[2]
                location = getLocationDescription(item[3])
                place = item[4]
                status = getStatusDescription(item[5])
                document.addLine(number, description, serialnumber, location, place, status)
            cursor.close()
        except Exception as error:
            mysql.connection.rollback()
            logging.error(str(error))
            return jsonify("error: " + str(error))
    document.addFooter("*** Ende der Liste ***")
    pdffile = document.createPDF()
    return pdffile

# -------------------------------------------------------------    
# count list
@app.route("/countlist")
def countlist():
    logging.info("countlist")
    checkSession()
    checkBooking()
    categorylist = getCategoryList()
    locationlist = getLocationList()
    return render_template("countlist.html", categorylist=categorylist, locationlist=locationlist, admin=session["admin"], info="", warning="")

@app.route("/printcountlist", methods=["POST"])
def printcountlist():
    logging.info("printcountlist")
    checkSession()
    location = request.form["location"]
    if location != "-":
        location = getLocationNumber(location)
    category = request.form["category"]
    if category != "-":
        category = getCategoryNumber(category)
        
    price = request.form["price"]
    document = itemlist.Liste()
    document.setPath(TempPath)
    document.setMarginPage(20, 20, 20, 20)
    currentdate = date.today()
    datestring = currentdate.strftime("%d.%m.%Y")
    document.setTitle("Zählliste vom " + datestring + " ab Wert " + str(price) +  " €                   Seite: ")
    document.addTab(1)
    document.addTab(8)
    document.addTab(30)
    document.addTab(45)
    document.addTab(65)
    document.addTab(70)
    document.addHeader("Num")
    document.addHeader("Bezeichnung")
    document.addHeader("Standort")
    document.addHeader("Platz")

    values = {}
    values["price"] = price
    cursor = mysql.connection.cursor()
    if location == "-" and category == "-":
        cursor.execute("SELECT number, description, serialnumber, location, place, status, quantity FROM item WHERE price >= %(price)s ORDER BY number", values)
    else:
        values = {}
        select = " WHERE"
        if location != "-":
            values["location"] = location
            select = select + " location = %(location)s"
        if category != "-":
            values["category"] = category
            if location != "":
                select = select + " AND"
            select = select + " category = %(category)s"
        cursor.execute("SELECT number, description, serialnumber, location, place, status, quantity FROM item " + select + " ORDER BY number", values)

    result = cursor.fetchall()
    for item in result:
        number = item[0]
        description = item[1]
        location = getLocationDescription(item[3])
        place = item[4]
        status = getStatusDescription(item[5])
        document.addLine(number, description, location, place, "", "")
        if item[6] > 1:
            document.addLineText("                   Anzahl: " + str(item[6]) + "         Aufgenommen: _________")
    cursor.close()
    document.addFooter("*** Ende der Liste ***")
    document.addFooter("")
    document.addFooter("")
    document.addFooter("Datum: ____________________  durchgeführt von: ___________________________________")
    pdffile = document.createPDF()
    return send_file(pdffile, as_attachment=True)

# -------------------------------------------------------------    
# print missing list
@app.route("/printmissinglist", methods=["POST"])
def printmissinglist():
    logging.info("printmissinglist")
    checkSession()
    checkBooking()
    inventdate = request.form["inventdate"]
    location = request.form["location"]
    if location != "-":
        location = getLocationNumber(location)
        
    document = itemlist.Liste()
    document.setPath(TempPath)
    document.setMarginPage(20, 20, 20, 20)
    currentdate = date.today()
    datestring = currentdate.strftime("%d.%m.%Y")
    inventdate = datetime.strptime(inventdate,'%Y-%m-%d')
    inventdatestring = inventdate.strftime("%d.%m.%Y")
    document.setTitle("Nicht inventiert vom " + datestring + "       Beginn Inventur: " + inventdatestring + "          Seite: ")
    document.addTab(1)
    document.addTab(8)
    document.addTab(30)
    document.addTab(45)
    document.addTab(55)
    document.addTab(70)
    document.addHeader("Num")
    document.addHeader("Bezeichnung")
    document.addHeader("Seriennummer")
    document.addHeader("Standort")
    document.addHeader("Platz")
    document.addHeader("Status")

    cursor = mysql.connection.cursor()
    values = {}
    values["lastinspection"] = inventdate
    logging.info(values)
    if location == "-":
        cursor.execute("SELECT number, description, serialnumber, location, place, status FROM item WHERE lastinspection < %(lastinspection)s ORDER BY number", values)
    else:
        values["location"] = location
        logging.info(values)
        cursor.execute("SELECT number, description, serialnumber, location, place, status FROM item WHERE lastinspection < %(lastinspection)s AND location = %(location)s ORDER BY number", values)

    result = cursor.fetchall()
    for item in result:
        number = item[0]
        description = item[1]
        serialnumber = item[2]
        location = getLocationDescription(item[3])
        place = item[4]
        status = getStatusDescription(item[5])
        document.addLine(number, description, serialnumber, location, place, status)
    cursor.close()
    document.addFooter("*** Ende der Liste ***")
    document.addFooter("")
    document.addFooter("")
    document.addFooter("Datum: ____________________  durchgeführt von: ___________________________________")
    pdffile = document.createPDF()
    return send_file(pdffile, as_attachment=True)

# -------------------------------------------------------------    
# electrical list
@app.route("/printelectricallist", methods=["POST"])
def printelectricallist():
    logging.info("printelectricallist")
    checkSession()
    checkBooking()
    checkdate = request.form["checkdate"]
    location = request.form["location"]
    if location != "-":
        location = getLocationNumber(location)
    document = itemlist.Liste()
    document.setPath(TempPath)
    document.setMarginPage(20, 20, 20, 20)
    currentdate = date.today()
    datestring = currentdate.strftime("%d.%m.%Y")
    document.setTitle("Liste DGUV3 vom " + datestring + "                                     Seite: ")
    document.addTab(1)
    document.addTab(8)
    document.addTab(25)
    document.addTab(40)
    document.addTab(50)
    document.addTab(65)
    document.addHeader("Num")
    document.addHeader("Bezeichnung")
    document.addHeader("Standort")
    document.addHeader("Platz")
    document.addHeader("Status")
    document.addHeader("")

    values = {}
    datestring = currentdate.strftime("%Y-%m-%d")
    values["datum"] = checkdate
    try:
        cursor = mysql.connection.cursor()
        if location == "-":
            cursor.execute("SELECT number, description, serialnumber, location, place, status, lastelectricaltest FROM item WHERE lastelectricaltest < %(datum)s ORDER BY number", values)
        else:
            values["location"] = location
            cursor.execute("SELECT number, description, serialnumber, location, place, status, lastelectricaltest FROM item WHERE location = %(location)s AND lastelectricaltest < %(datum)s ORDER BY number", values)
    
        result = cursor.fetchall()
        for item in result:
            number = item[0]
            description = item[1]
            serialnumber = item[2]
            location = getLocationDescription(item[3])
            place = item[4]
            status = getStatusDescription(item[5])
            datum = item[6]
            pruefdatum = datum.strftime("%d.%m.%Y")
            document.addLine(number, description, location, place, status, "")
            document.addLine("", "letzte Prüfung:", pruefdatum, "", "", "")
            
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    document.addFooter("*** Ende der Liste ***")
    document.addFooter("")
    document.addFooter("")
    document.addFooter("Datum: ____________________  erledigt von: ___________________________________")
    pdffile = document.createPDF()
    return send_file(pdffile, as_attachment=True)

# -------------------------------------------------------------    
# all list
def alllist():
    logging.info("alllist")
    checkSession()
    return render_template("alllist.html",  admin=session["admin"], info="", warning="")

@app.route("/printalllist", methods=["POST"])
def printalllist():
    logging.info("printalllist")
    checkSession()
    checkBooking()
    document = itemlist.Liste()
    document.setPath(TempPath)
    document.setMarginPage(20, 20, 20, 20)
    currentdate = date.today()
    datestring = currentdate.strftime("%d.%m.%Y")
    document.setTitle("Gesamtliste vom " + datestring + "                                     Seite: ")
    document.addTab(1)
    document.addTab(8)
    document.addTab(30)
    document.addTab(45)
    document.addTab(55)
    document.addTab(70)
    document.addHeader("Num")
    document.addHeader("Bezeichnung")
    document.addHeader("Standort")
    document.addHeader("Platz")
    document.addHeader("Status")
    document.addHeader("Letze Inventur")

    orderby = request.form.getlist("sortby")
    values = {}
    values["orderby"] = orderby
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number, description, location, place, status, lastinspection FROM item ORDER BY %(orderby)s", values)
    
        result = cursor.fetchall()
        for item in result:
            number = item[0]
            description = item[1]
            location = getLocationDescription(item[2])
            place = item[3]
            status = getStatusDescription(item[4])
            lastinspection = item[5]
            document.addLine(number, description, location, place, status, lastinspection)
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    document.addFooter("*** Ende der Liste ***")
    pdffile = document.createPDF()
    return send_file(pdffile, as_attachment=True)

# -------------------------------------------------------------    
# user
@app.route("/user")
def user():
    logging.info("user")
    checkSession()
    checkAdmin()
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT number, username, admin, disabled FROM user")
    result = cursor.fetchall()
    userlist = []
    for line in result:
        user = []
        user.append(line[0])
        user.append(line[1])
        
        if line[2] == 1:
            user.append("checked")
        else:
            user.append("")
        if line[3] == 1:
            user.append("checked")
        else:
            user.append("")
        userlist.append(user)
    cursor.close()
    return render_template("user.html", userlist=userlist, admin=session["admin"], info="", warning="")

# edit user
@app.route("/useredit", methods=["POST"])
def useredit():
    logging.info("useredit")
    checkSession()
    checkAdmin()
    action = ""
    
    try:
        value = request.form["edit"]
        action = "edit"
    except:
        try:
            value = request.form["delete"]
            action = "delete"
        except: 
            value = request.form["new"]
            action = "new"
    
    if action == "new":
        return render_template("usernew.html", admin=session["admin"], info="", warning="")

    values = {}
    values["number"] = value
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT number, username, admin, disabled, movement FROM user WHERE number= %(number)s", values)
    result = cursor.fetchall()
    for line in result:
        number = line[0]
        username = line[1]
        adminuser = ""
        if line[2] == 1:
            adminuser = "checked"
        disabled = ""
        if line[3] == 1:
            disabled = "checked"
        movement = ""
        if line[4] == 1:
            movement = "checked"
    login = False
    cursor.close()
    if action == "delete":
        return render_template("userdelete.html", number=number, username=username, adminuser=adminuser, disabled=disabled, movement=movement, admin=session["admin"], info="", warning="")
    logging.info(movement)
    return render_template("useredit.html", number=number, username=username, adminuser=adminuser, disabled=disabled, movement=movement, admin=session["admin"], info="", warning="")
        
# new user
@app.route("/usersavenew", methods=["POST"])
def usersavenew():
    logging.info("usersavenew")
    checkSession()
    checkAdmin()
    values = []
    values.append(request.form["username"])
    values.append(encodePassword(request.form["password"]))
    valueadmin = request.form.get("adminuser", default=False, type=bool)
    if valueadmin:
        values.append(1)
    else:
        values.append(0)
    valuedisabled = request.form.get("disabled", default=False, type=bool)
    if valuedisabled:
        values.append(1)
    else:
        values.append(0)
    valuemovement = request.form.get("movement", default=False, type=bool)
    if valuemovement:
        values.append(1)
    else:
        values.append(0)
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("INSERT INTO user (username, password, admin, disabled, movement) VALUES (%s, %s, %s, %s, %s)", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect("/user")

# save edit user
@app.route("/usersave", methods=["POST"])
def usersave():
    logging.info("usersave")
    checkSession()
    checkAdmin()
    values = []
    values.append(encodePassword(request.form["password"]))
    valueadmin = request.form.get("adminuser", default=False, type=bool)
    if valueadmin:
        values.append(1)
    else:
        values.append(0)
    valuedisabled = request.form.get("disabled", default=False, type=bool)
    if valuedisabled:
        values.append(1)
    else:
        values.append(0)
    valuemovement = request.form.get("movement", default=False, type=bool)
    if valuemovement:
        values.append(1)
    else:
        values.append(0)
    values.append(request.form["number"])
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("UPDATE user SET password = %s, admin = %s, disabled = %s, movement = %s WHERE number = %s", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect ("/user")

# delete user
@app.route("/userdelete", methods=["POST"])
def userdelete():
    logging.info("userdelete")
    checkSession()
    checkAdmin()
    values = []
    values.append(request.form["number"])
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("DELETE FROM user WHERE number = %s", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect ("/user")

# -------------------------------------------------------------    
# maintenance
@app.route("/maintenance")
def maintenance():
    logging.info("maintenance")
    checkSession()
    return render_template("maintenance.html", admin=session["admin"], info="", warning="")

@app.route("/maintenancechoose", methods=["POST"])
def maintenancechoose():
    logging.info("maintenancechoose")
    checkSession()
    try:
        value = request.form["password"]
        action = "password"
    except:
        pass
    try:
        value = request.form["ftp"]
        action = "ftp"
    except:
        pass

    if action == "password":
        cursor = mysql.connection.cursor()
        cursor.execute("SELECT number, username FROM user")
        result = cursor.fetchall()
        for line in result:
            number = line[0]
            username = line[1]
        cursor.close()
        return render_template("password.html", number=number, username=username, admin=session["admin"], info="", warning="")
    if action == "ftp":
        hostname = socket.gethostname()
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(("10.255.255.255", 1))
        ip = s.getsockname()[0]
        return render_template("ftp.html", ip=ip, admin=session["admin"], info="", warning="")        
    return redirect ("/maintenance")

# save user password
@app.route("/maintenancesave", methods=["POST"])
def maintenancesave():
    logging.info("maintenancesave")
    checkSession()
    values = []
    values.append(encodePassword(request.form["password"]))
    values.append(request.form["number"])
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("UPDATE user SET password = %s WHERE number = %s", values)
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect ("/search")

# save ftp password
@app.route("/ftpsave", methods=["POST"])
def ftpsave():
    logging.info("ftpsave")
    checkSession()
    password = request.form["password"]
    try:
        result = os.system("sudo setpasswd.sh '" + password + "' > /tmp/setpasswd.mld 2>&1")
        if result != 0:
            logging.error(str(result))
            return jsonify("error: " + str(result))
    except Exception as error:
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect ("/search")

# -------------------------------------------------------------    
# lists
@app.route("/list")
def list():
    logging.info("list")
    checkSession()
    checkBooking()
    return render_template("list.html", admin=session["admin"], info="", warning="")

@app.route("/listchoose", methods=["POST"])
def listchoose():
    logging.info("listchoose")
    checkSession()
    checkBooking()
    try:
        value = request.form["countlist"]
        action = "countlist"
    except:
        pass
    try:
        value = request.form["missinglist"]
        action = "missinglist"
    except:
        pass
    try:
        value = request.form["electricallist"]
        action = "electricallist"
    except:
        pass
    try:
        value = request.form["alllist"]
        action = "alllist"
    except:
        pass
    
    if action == "countlist":
        categorylist = getCategoryList()
        locationlist = getLocationList()
        return render_template("countlist.html", categorylist=categorylist, locationlist=locationlist, admin=session["admin"], info="", warning="")
    if action == "missinglist":
        inventdate = date.today()
        locationlist = getLocationList()
        return render_template("missinglist.html", locationlist=locationlist, inventdate=inventdate, admin=session["admin"], info="", warning="")
    if action == "electricallist":
        locationlist = getLocationList()
        checkdate = date.today()
        return render_template("electricallist.html", locationlist=locationlist, checkdate=checkdate, admin=session["admin"], info="", warning="")
    if action == "alllist":
        return render_template("alllist.html", admin=session["admin"], info="", warning="")
    return redirect ("/list")

# -------------------------------------------------------------    
# counter
@app.route("/counter")
def counter():
    logging.info("counter")
    checkSession()
    checkAdmin()
    cursor = mysql.connection.cursor()
    cursor.execute("SELECT item FROM counter WHERE number = 1")
    result = cursor.fetchall()
    for line in result:
        counter = line[0]
    cursor.close()
    return render_template("counter.html", counter=counter, admin=session["admin"], info="", warning="")

# save counter
@app.route("/countersave", methods=["POST"])
def countersave():
    logging.info("countersave")
    checkSession()
    checkAdmin()
    values = []
    counter = int(request.form["counter"])
    values.append(counter)
    try:
        cursor = mysql.connection.cursor()
        cursor.execute("LOCK TABLES counter WRITE");
        cursor.execute("SELECT item FROM counter WHERE number = 1")
        result = cursor.fetchall()
        for line in result:
            counterdb = int(line[0])
        if counter <= counterdb:
            warning = "Eingabe kleiner als aktueller Zähler"
            return render_template("counter.html", counter=counterdb, admin=session["admin"], info="", warning=warning)
        cursor.execute("UPDATE counter SET item = %s WHERE number = 1", values)
        cursor.execute("UNLOCK TABLES");
        mysql.connection.commit()
        cursor.close()
    except Exception as error:
        mysql.connection.rollback()
        logging.error(str(error))
        return jsonify("error: " + str(error))
    return redirect ("/search")

# -------------------------------------------------------------    
@app.route("/logout")
def logout():
    logging.info("logout")
    session.clear()
    return redirect("/login")

# -----------------------------------------------------------------------
def checkSession():
    logging.info("checksession")
    if "username" in session:
        return
    return abort(403, "Sie sind nicht angemeldet")

def checkAdmin():
    if "admin" in session:
        if not session["admin"]:
            return abort(403, "Sie haben nicht die notwendigen Rechte fuer diese Aktion")

def checkBooking():
    if "movement" in session:
        if session["movement"]:
            return abort(403, "Sie haben nicht die notwendigen Rechte fuer diese Aktion")

if __name__=="__main__":
    app.run(debug=True, threaded=True)
