# -*- coding: utf-8 -*-
from reportlab.pdfgen import canvas
from reportlab.lib.pagesizes import A4
from reportlab.lib.units import cm
from reportlab.graphics import renderPDF
from reportlab.pdfbase.pdfmetrics import stringWidth
import tempfile
import os
import logging

class Liste():
    Faktor = 2.845
    WidthA4 = 595.2
    HeightA4 = 841.8
    SpaceY = 2.845
    SpaceX = 2.845
    SizeY = 12
    MarginPageLeft = 0
    MarginPageRight = 0
    MarginPageTop = 0
    MarginPageBottom = 0
    # Margin label 4 mm
    MarginX = 11.4 
    MarginY = 11.4

    def __init__(self):
        self.Label = []
        self.Footer = []
        self.Tab = []
        self.Header = []
        self.Page = 0

    def setMarginPage(self, left, right, top, bottom):
        self.MarginPageLeft = left * self.Faktor
        self.MarginPageRight = right * self.Faktor
        self.MarginPageTop = top * self.Faktor
        self.MarginPageBottom = bottom * self.Faktor
    
    def setPath(self, path):
        self.Path = path
        
    def setTitle(self, title):
        self.Title = title

    def addHeader(self, header):
        self.Header.append(header)
        
    def addTab(self, tab):
        self.Tab.append(tab)
    
    def addLine(self, value1, value2, value3, value4, value5, value6):
        label = []
        label.append(str(value1))
        label.append(str(value2))
        label.append(str(value3))
        label.append(str(value4))
        label.append(str(value5))
        label.append(str(value6))
        self.Label.append(label)

    def addLineText(self, value):
        label = []
        label.append(value)
        self.Label.append(label)
    
    def addFooter(self, footer):
        self.Footer.append(footer)
        
    def writeTitle(self, pdf, xtext, ytext):
        self.Page = self.Page + 1
        pdf.setFont("Helvetica",14)
        pdf.drawString(xtext, ytext, self.Title + str(self.Page))
        pdf.setFont("Helvetica",11)
        charWidth = stringWidth("X", "Helvetica", 11)
        # write Header
        ytext = ytext - self.SizeY * 2
        index = 0
        for text in self.Header:
            xtext = self.MarginPageLeft + charWidth * self.Tab[index]
            pdf.drawString(xtext, ytext, text)
            index = index + 1
        pdf.line(self.MarginPageLeft, ytext - self.SizeY, self.WidthA4 - self.MarginPageRight, ytext - self.SizeY)
        ytext = ytext - self.SizeY * 3
        return ytext
        
    def createPDF(self):
        temp = tempfile.NamedTemporaryFile(dir=self.Path, prefix="list_", suffix=".pdf", delete=False)
        filename = temp.name
        pdf = canvas.Canvas(filename, pagesize=A4)
        pdf.setFont("Helvetica",11)
        charWidth = stringWidth("X", "Helvetica", 11)
        
        top = True
        for label in self.Label:
            if top:
                ytext = self.HeightA4 - self.MarginY - self.MarginPageTop - self.SizeY
                xtext = self.MarginPageLeft
                ytext = self.writeTitle(pdf, xtext, ytext)
                top = False

            index = 0
            for text in label:
                if index < len(label) - 1:
                    length = self.Tab[index + 1] - self.Tab[index] - 1
                    textlength = len(text)
                    logging.info(length)
                    logging.info(textlength)
                    if textlength > length:
                        text = text[0:length]
                xtext = self.MarginPageLeft + charWidth * self.Tab[index]
                pdf.drawString(xtext, ytext, text)
                index = index + 1

            xtext = self.MarginPageLeft
            ytext = ytext - self.SizeY - self.SpaceY
            
            if ytext < self.SizeY:
                pdf.showPage()
                top = True

        xtext = self.MarginPageLeft
        index = 0
        for text in self.Footer:
            if top:
                ytext = self.HeightA4 - self.MarginY - self.MarginPageTop - self.SizeY
                ytext = self.writeTitle(pdf, xtext, ytext)
                top = False
                
            pdf.drawString(xtext, ytext, text)
            index = index + 1
            ytext = ytext - self.SizeY - self.SpaceY
            
            if ytext < self.SizeY:
                pdf.showPage()
                top = True
            
        pdf.save()
        return filename

if __name__ == "__main__":
    document = Liste()
    document.setMarginPage(20, 20, 20, 20)
    document.setPath("/work/volker/daten/temp/")
    document.setTitle("Inventurliste            Seite: ")
    document.addTab(1)
    document.addTab(8)
    document.addTab(30)
    document.addTab(45)
    document.addTab(55)
    document.addTab(70)
    document.addHeader("Num")
    document.addHeader("Bezeichnung")
    document.addHeader("Seriennummer")
    document.addHeader("Standort")
    document.addHeader("Platz")
    document.addHeader("Status")
    document.addLine("1001", "Monitor", "SN123456", "Karlsruhe", "Büro VM", "im Einsatz")
    document.addLine("1002", "Tisch", "SNABC56", "Karlsruhe", "Büro VM", "im Einsatz")
    pdf = document.createPDF()
    print(pdf)
