# -*- coding: utf-8 -*-
from reportlab.pdfgen import canvas
from reportlab.lib.pagesizes import A4
from reportlab.lib.units import cm
from reportlab.graphics import renderPDF
from svglib.svglib import svg2rlg
from barcode import Code39
import tempfile
import os
import logging
# 1 Punkt = 1/72 inch = 
class Label():
    WidthA4 = 595.2
    HeightA4 = 841.8
    MarginPageLeft = 0
    MarginPageRight = 0
    MarginPageTop = 0
    MarginPageBottom = 0
    # Margin label 2 mm
    MarginX = 5.7
    MarginY = 5.7
    SizeX = 0
    SizeY = 0
    #Faktor = 2.845
    #SpaceX = 28.45
    #SpaceY = 2.845
    Faktor = 2.835
    SpaceX = 28.34
    SpaceY = 2.835
    ScaleWidth = 1
    ScaleHeight = 1
    StartRow = 1
    StartColumn = 1

    def __init__(self, filename=""):
        self.Label = []
        self.Filename = filename
        
    # convert svg for PDF
    def convertsvg(self, imagepath):
        drawing = svg2rlg(imagepath)
        scalewidth = self.ScaleWidth
        scaleheight = self.ScaleHeight
        drawing.scale(scalewidth, scaleheight)
        return drawing

    def createBarcode(self, text):
        code = Code39(text, add_checksum=False)
        # write barcode to svg file
        temp = tempfile.NamedTemporaryFile(dir=self.Path, prefix="barcode_", delete=False)
        filename = temp.name
        code.save(filename)
        bc = self.convertsvg(filename + ".svg")
        return bc
        
    def setMarginPage(self, left, right, top, bottom):
        self.MarginPageLeft = left * self.Faktor
        self.MarginPageRight = right * self.Faktor
        self.MarginPageTop = top * self.Faktor
        self.MarginPageBottom = bottom * self.Faktor
    
    def setSize(self, x, y):
        self.SizeX = x * self.Faktor
        self.SizeY = y * self.Faktor
    
    def setMargin(self, x, y):
        self.MarginX = x * self.Faktor
        self.MarginY = y * self.Faktor
        
    def setPage(self, size):
        self.PageSize = size
        
    def setPath(self, path):
        self.Path = path
        
    def setScale(self, scalewidth, scaleheight):
        self.ScaleWidth = scalewidth
        self.ScaleHeight = scaleheight
    
    def setStart(self, row, column):
        if row > 0:
            self.StartRow = row
        if column > 0:
            self.StartColumn = column
        
    def addLabel(self, text1, text2, barcode):
        label = []
        ttext1 = str(text1)
        label.append(ttext1)
        ttext2 = str(text2)
        label.append(ttext2)
        tbarcode = str(barcode)
        label.append(tbarcode)
        self.Label.append(label)
    
    def createPDF(self):
        if self.Filename == "":
            temp = tempfile.NamedTemporaryFile(dir=self.Path, prefix="label_", suffix=".pdf", delete=False)
            filename = temp.name
        else:
            filename = self.Path + self.Filename
        pdf = canvas.Canvas(filename, pagesize=A4)
        pdf.setFont("Helvetica",12)
        
        x = self.MarginPageLeft
        xtext = x
        y = self.HeightA4
        
        if self.StartColumn > 1:
            xtext = x + ((self.SizeX + self.MarginX) * (self.StartColumn - 1))
        if self.StartRow > 1:
            y = y - ((self.SizeY + self.MarginY) * (self.StartRow - 1))
            
        top = True
        for label in self.Label:
            text1 = str(label[0])
            text2 = str(label[1])
            bctext = str(label[2])
        
            if top:
                y = y - self.MarginPageTop
                top = False
                
            ytext = y - self.MarginY - self.SpaceY * 2
            pdf.drawString(xtext, ytext, text1)
            
            if text2 != "":
                ytext = ytext - self.SpaceY * 6
                pdf.drawString(xtext, ytext, text2)
            
            bc = self.createBarcode(bctext)
            ytext = ytext - self.SpaceY * 26 * self.ScaleHeight
            renderPDF.draw(bc, pdf, xtext, ytext)
            
            xtext = xtext + self.SizeX + self.MarginX
            if xtext + self.SizeX > self.WidthA4:
                x = self.MarginPageLeft
                xtext = x
                y = y - self.SizeY - self.MarginY
                if ytext < self.SizeY + self.MarginPageBottom:
                    pdf.showPage()
                    x = self.MarginPageLeft
                    xtext = x
                    y = self.HeightA4
                    top = True

        pdf.save()
        self.deleteFiles()
        return filename
    
    def deleteFiles(self):
        os.chdir(self.Path)
        for file in os.listdir("."):
            if os.path.isfile(file) and file.startswith("barcode_"):
                try:
                    os.remove(file)
                except Exception as Error:
                    pass


if __name__ == "__main__":
    document = Label(filename="test.pdf")
    document.setMarginPage(10, 0, 20, 10)
    document.setScale(1, 0.5)
    document.setSize(45.7, 21.2)
    document.setMargin(4, 1)
    document.setPath("/work/volker/daten/temp/")
    document.addLabel("Stiftung-Forum-Recht", "", "111111")
    document.addLabel("Stiftung-Forum-Recht", "", "222222")
    document.addLabel("Stiftung-Forum-Recht", "", "333333")
    document.addLabel("Stiftung-Forum-Recht", "", "444444")
    document.addLabel("Stiftung-Forum-Recht", "", "555555")
    document.addLabel("Stiftung-Forum-Recht", "", "666666")
    document.addLabel("Stiftung-Forum-Recht", "", "777777")
    document.addLabel("Stiftung-Forum-Recht", "", "888888")
    document.addLabel("Stiftung-Forum-Recht", "", "111111")
    document.addLabel("Stiftung-Forum-Recht", "", "222222")
    document.addLabel("Stiftung-Forum-Recht", "", "333333")
    document.addLabel("Stiftung-Forum-Recht", "", "444444")
    document.addLabel("Stiftung-Forum-Recht", "", "555555")
    document.addLabel("Stiftung-Forum-Recht", "", "666666")
    document.addLabel("Stiftung-Forum-Recht", "", "777777")
    document.addLabel("Stiftung-Forum-Recht", "", "888888")
    document.addLabel("Stiftung-Forum-Recht", "", "111111")
    document.addLabel("Stiftung-Forum-Recht", "", "222222")
    document.addLabel("Stiftung-Forum-Recht", "", "333333")
    document.addLabel("Stiftung-Forum-Recht", "", "444444")
    document.addLabel("Stiftung-Forum-Recht", "", "555555")
    document.addLabel("Stiftung-Forum-Recht", "", "666666")
    document.addLabel("Stiftung-Forum-Recht", "", "777777")
    document.addLabel("Stiftung-Forum-Recht", "", "888888")    
    document.addLabel("Stiftung-Forum-Recht", "", "111111")
    document.addLabel("Stiftung-Forum-Recht", "", "222222")
    document.addLabel("Stiftung-Forum-Recht", "", "333333")
    document.addLabel("Stiftung-Forum-Recht", "", "444444")
    document.addLabel("Stiftung-Forum-Recht", "", "555555")
    document.addLabel("Stiftung-Forum-Recht", "", "666666")
    document.addLabel("Stiftung-Forum-Recht", "", "777777")
    document.addLabel("Stiftung-Forum-Recht", "", "888888")
    document.addLabel("Stiftung-Forum-Recht", "", "111111")
    document.addLabel("Stiftung-Forum-Recht", "", "222222")
    document.addLabel("Stiftung-Forum-Recht", "", "333333")
    document.addLabel("Stiftung-Forum-Recht", "", "444444")
    document.addLabel("Stiftung-Forum-Recht", "", "555555")
    document.addLabel("Stiftung-Forum-Recht", "", "666666")
    document.addLabel("Stiftung-Forum-Recht", "", "777777")
    document.addLabel("Stiftung-Forum-Recht", "", "888888")
    document.addLabel("Stiftung-Forum-Recht", "", "111111")
    document.addLabel("Stiftung-Forum-Recht", "", "222222")
    document.addLabel("Stiftung-Forum-Recht", "", "333333")
    document.addLabel("Stiftung-Forum-Recht", "", "444444")
    document.addLabel("Stiftung-Forum-Recht", "", "555555")
    document.addLabel("Stiftung-Forum-Recht", "", "666666")
    document.addLabel("Stiftung-Forum-Recht", "", "777777")
    document.addLabel("Stiftung-Forum-Recht", "", "888888")    
    document.addLabel("Stiftung-Forum-Recht", "", "111111")
    document.addLabel("Stiftung-Forum-Recht", "", "222222")
    document.addLabel("Stiftung-Forum-Recht", "", "333333")
    document.addLabel("Stiftung-Forum-Recht", "", "444444")
    document.addLabel("Stiftung-Forum-Recht", "", "555555")
    document.addLabel("Stiftung-Forum-Recht", "", "666666")
    document.addLabel("Stiftung-Forum-Recht", "", "777777")
    document.addLabel("Stiftung-Forum-Recht", "", "888888")
    document.addLabel("Stiftung-Forum-Recht", "", "111111")
    document.addLabel("Stiftung-Forum-Recht", "", "222222")
    document.addLabel("Stiftung-Forum-Recht", "", "333333")
    document.addLabel("Stiftung-Forum-Recht", "", "444444")
    document.addLabel("Stiftung-Forum-Recht", "", "555555")
    document.addLabel("Stiftung-Forum-Recht", "", "666666")
    document.addLabel("Stiftung-Forum-Recht", "", "777777")
    document.addLabel("Stiftung-Forum-Recht", "", "888888")
    document.addLabel("Stiftung-Forum-Recht", "", "111111")
    document.addLabel("Stiftung-Forum-Recht", "", "222222")
    document.addLabel("Stiftung-Forum-Recht", "", "333333")
    document.addLabel("Stiftung-Forum-Recht", "", "444444")
    document.addLabel("Stiftung-Forum-Recht", "", "555555")
    document.addLabel("Stiftung-Forum-Recht", "", "666666")
    document.addLabel("Stiftung-Forum-Recht", "", "777777")
    document.addLabel("Stiftung-Forum-Recht", "", "888888")        
    pdf = document.createPDF()
    print(pdf)
